from .base import ExordiumUserTests

from django.urls import reverse
from django.db.models import Q

from django.templatetags.static import static

from exordium.models import Artist, Album, Song, App, AlbumArt

class IndexViewTests(ExordiumUserTests):
    """
    Tests of our main index view.  (Not a whole lot going on, really)
    """

    def test_no_albums(self):
        """
        Test what happens if there are no albums defined.  (No albums should
        be found in the view's context.)  This test also tests for the various
        links which are supposed to be present on the main page.
        """
        response = self.client.get(reverse('exordium:index'))
        self.assertEqual(response.status_code, 200)
        self.assertQuerysetEqual(response.context['album_list'].data, [])

        self.assertContains(response, 'AnonymousUser')
        self.assertContains(response, reverse('exordium:browse_artist'))
        self.assertContains(response, reverse('exordium:browse_album'))
        self.assertContains(response, reverse('exordium:updateprefs'))

        # We are not authenticated, so we should NOT see any of the admin
        # links anywhere.
        self.assertNotContains(response, reverse('exordium:library'))
        self.assertNotContains(response, reverse('admin:dynamic_preferences_globalpreferencemodel_changelist'))
        self.assertNotContains(response, '%s"' % (reverse('admin:index')))

        # But we SHOULD see a login link
        self.assertContains(response, reverse('admin:login'))

    def test_no_albums_admin(self):
        """
        Test what happens if there are no albums defined.  (No albums should
        be found in the view's context.)  This test also tests for the various
        links which are supposed to be present on the main page.  This time,
        request as if we're an admin.
        """
        self.login()
        response = self.client.get(reverse('exordium:index'))
        self.assertContains(response, 'mainuser')
        self.assertEqual(response.status_code, 200)
        self.assertQuerysetEqual(response.context['album_list'].data, [])

        self.assertContains(response, 'mainuser')
        self.assertContains(response, reverse('exordium:browse_artist'))
        self.assertContains(response, reverse('exordium:browse_album'))
        self.assertContains(response, reverse('exordium:updateprefs'))

        # We are authenticated, so we should see the admin links.
        self.assertContains(response, reverse('exordium:library'))
        self.assertContains(response, reverse('admin:dynamic_preferences_globalpreferencemodel_changelist'))
        self.assertContains(response, reverse('admin:index'))

        # But we should NOT see a login link
        self.assertNotContains(response, reverse('admin:login'))

    def test_single_album(self):
        """
        Test what happens when there's a single album.  We should see that
        album!
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Test Album', filename='song1.mp3')
        self.run_add()
        self.assertEqual(Album.objects.count(), 1)
        album = Album.objects.get()

        response = self.client.get(reverse('exordium:index'))
        self.assertEqual(response.status_code, 200)
        self.assertQuerysetEqual(response.context['album_list'].data, [repr(album)])
        self.assertContains(response, 'Test Album')
        self.assertContains(response, reverse('exordium:album', args=(album.pk,)))
        self.assertContains(response, reverse('exordium:artist', args=(album.artist.normname,)))
        self.assertContains(response, '1 album')

        # May as well verify that we're seeing the no-album-art cover, as well
        self.assertContains(response, '"%s"' % (static('exordium/no_album_art_small.png')))

    def test_album_art(self):
        """
        Make sure that an album with album art has its art showing in the table.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Test Album', filename='song1.mp3')
        self.add_art()
        self.run_add()
        self.assertEqual(Album.objects.count(), 1)
        album = Album.objects.get()

        response = self.client.get(reverse('exordium:index'))
        self.assertEqual(response.status_code, 200)
        self.assertQuerysetEqual(response.context['album_list'].data, [repr(album)])
        self.assertContains(response, 'Test Album')
        self.assertContains(response, reverse('exordium:album', args=(album.pk,)))
        self.assertContains(response, reverse('exordium:artist', args=(album.artist.normname,)))
        self.assertContains(response, reverse('exordium:albumart', args=(album.pk, 'list',)))

    def test_four_albums(self):
        """
        Test what happens when there's four albums.  Ensure that they are
        sorted properly.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album 1', filename='song1.mp3')
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album 2', filename='song2.mp3')
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album 3', filename='song3.mp3')
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album 4', filename='song4.mp3')
        self.run_add()
        album_2 = self.age_album('Artist', 'Album 2', 2)
        album_3 = self.age_album('Artist', 'Album 3', 4)
        album_4 = self.age_album('Artist', 'Album 4', 6)
        self.assertEqual(Album.objects.count(), 4)
        album_1 = Album.objects.get(name='Album 1')

        response = self.client.get(reverse('exordium:index'))
        self.assertEqual(response.status_code, 200)
        self.assertQuerysetEqual(response.context['album_list'].data,
            [repr(al) for al in [album_1, album_2, album_3, album_4]])
        self.assertContains(response, 'Album 1')
        self.assertContains(response, 'Album 2')
        self.assertContains(response, 'Album 3')
        self.assertContains(response, 'Album 4')
        self.assertContains(response, '4 albums')

    def test_classical_album(self):
        """
        Test the display of an album with classical tags
        """
        self.add_mp3(artist='Artist', title='Title 1',
            group='Group', conductor='Conductor', composer='Composer',
            album='Album 1', filename='song1.mp3')
        self.run_add()

        self.assertEqual(Artist.objects.count(), 5)
        self.assertEqual(Album.objects.count(), 1)

        album = Album.objects.get()
        artists = []

        response = self.client.get(reverse('exordium:index'))
        self.assertEqual(response.status_code, 200)
        self.assertQuerysetEqual(response.context['album_list'].data, [repr(album)])
        self.assertContains(response, 'Album 1')
        self.assertContains(response, reverse('exordium:album', args=(album.pk,)))
        self.assertContains(response, reverse('exordium:artist', args=(album.artist.normname,)))
        for artist in Artist.objects.exclude(name='Various'):
            self.assertContains(response, str(artist))
            self.assertContains(response, reverse('exordium:artist', args=(artist.normname,)))

    def test_classical_album_two_tracks(self):
        """
        Test the display of an album with classical tags.  Two tracks, where
        the artist stays the same but the other classical tags differ.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            group='Group', conductor='Conductor', composer='Composer',
            album='Album 1', filename='song1.mp3')
        self.add_mp3(artist='Artist', title='Title 2',
            group='Group 2', conductor='Conductor 2', composer='Composer 2',
            album='Album 1', filename='song2.mp3')
        self.run_add()

        self.assertEqual(Artist.objects.count(), 8)
        self.assertEqual(Album.objects.count(), 1)

        album = Album.objects.get()
        artists = []

        response = self.client.get(reverse('exordium:index'))
        self.assertEqual(response.status_code, 200)
        self.assertQuerysetEqual(response.context['album_list'].data, [repr(album)])
        self.assertContains(response, 'Album 1')
        self.assertContains(response, reverse('exordium:album', args=(album.pk,)))
        self.assertContains(response, reverse('exordium:artist', args=(album.artist.normname,)))
        for artist in Artist.objects.exclude(name='Various'):
            self.assertContains(response, str(artist))
            self.assertContains(response, reverse('exordium:artist', args=(artist.normname,)))

    def test_various_album(self):
        """
        Test display of a Various Artists album
        """
        self.add_mp3(artist='Artist 1', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_mp3(artist='Artist 2', title='Title 2',
            album='Album', filename='song2.mp3')
        self.run_add()

        self.assertEqual(Artist.objects.count(), 3)
        self.assertEqual(Album.objects.count(), 1)
        artist_1 = Artist.objects.get(name='Artist 1')
        artist_2 = Artist.objects.get(name='Artist 2')
        various = Artist.objects.get(name='Various')
        album = Album.objects.get()

        response = self.client.get(reverse('exordium:index'))
        self.assertEqual(response.status_code, 200)
        self.assertQuerysetEqual(response.context['album_list'].data, [repr(album)])
        self.assertContains(response, 'Album')
        self.assertContains(response, reverse('exordium:album', args=(album.pk,)))
        self.assertContains(response, reverse('exordium:artist', args=(album.artist.normname,)))
        self.assertContains(response, reverse('exordium:artist', args=(various.normname,)))
        for artist in [artist_1, artist_2]:
            self.assertNotContains(response, str(artist_1))
            self.assertNotContains(response, reverse('exordium:artist', args=(artist.normname,)))

    def test_classical_album_two_tracks_various(self):
        """
        Test the display of an album with classical tags.  Two tracks, where
        all artist fields change, so it's a Various Artists album.
        """
        self.add_mp3(artist='Artist 1', title='Title 1',
            group='Group', conductor='Conductor', composer='Composer',
            album='Album 1', filename='song1.mp3')
        self.add_mp3(artist='Artist 2', title='Title 2',
            group='Group 2', conductor='Conductor 2', composer='Composer 2',
            album='Album 1', filename='song2.mp3')
        self.run_add()

        self.assertEqual(Artist.objects.count(), 9)
        self.assertEqual(Album.objects.count(), 1)

        album = Album.objects.get()
        artists = []

        various = Artist.objects.get(name='Various')

        response = self.client.get(reverse('exordium:index'))
        self.assertEqual(response.status_code, 200)
        self.assertQuerysetEqual(response.context['album_list'].data, [repr(album)])
        self.assertContains(response, 'Album 1')
        self.assertContains(response, reverse('exordium:album', args=(album.pk,)))
        self.assertContains(response, reverse('exordium:artist', args=(album.artist.normname,)))
        self.assertContains(response, reverse('exordium:artist', args=(various.normname,)))
        for artist in Artist.objects.exclude(Q(name='Artist 1') | Q(name='Artist 2')):
            self.assertContains(response, str(artist))
            self.assertContains(response, reverse('exordium:artist', args=(artist.normname,)))
        for artist in Artist.objects.filter(Q(name='Artist 1') | Q(name='Artist 2')):
            self.assertNotContains(response, str(artist))
            self.assertNotContains(response, reverse('exordium:artist', args=(artist.normname,)))

    def test_pagination(self):
        """
        Test to make sure that our pagination is working properly.
        The index page will show a total of 20 albums
        """
        for num in range(30):
            self.add_mp3(artist='Artist', title='Title %d' % (num+1),
                album='Album %d' % (num+1), filename='song%d.mp3' % (num+1))
        self.run_add()
        self.assertEqual(Album.objects.count(), 30)

        albums = {}
        for num in range(30):
            self.age_album(artist='Artist',
                album='Album %d' % (num+1),
                age_days=num+1)
            albums[num] = Album.objects.get(name='Album %d' % (num+1))

        response = self.client.get(reverse('exordium:index'))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['album_list'].data), 30)
        for num in range(20):
            self.assertContains(response, '%s<' % (albums[num]))
            self.assertContains(response, reverse('exordium:album', args=(albums[num].pk,)))
        for num in range(20, 30):
            self.assertNotContains(response, '%s<' % (albums[num]))
            self.assertNotContains(response, reverse('exordium:album', args=(albums[num].pk,)))
        self.assertContains(response, '20 of 30 albums')
        self.assertContains(response, '"?page=2"')

        # test page 2
        response = self.client.get(reverse('exordium:index'), {'page': 2})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['album_list'].data), 30)
        for num in range(20):
            self.assertNotContains(response, '%s<' % (albums[num]))
            self.assertNotContains(response, reverse('exordium:album', args=(albums[num].pk,)))
        for num in range(20, 30):
            self.assertContains(response, '%s<' % (albums[num]))
            self.assertContains(response, reverse('exordium:album', args=(albums[num].pk,)))
        self.assertContains(response, '10 of 30 albums')
        self.assertContains(response, '"?page=1"')

    def test_sorting(self):
        """
        Test for at least one sorting link
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album 1', filename='song1.mp3', year=2016)
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album 2', filename='song2.mp3', year=2006)
        self.run_add()

        self.assertEqual(Album.objects.count(), 2)
        album_2 = self.age_album('Artist', 'Album 2', 2)
        album_1 = Album.objects.get(name='Album 1')

        # Initial view, should be sorted by addition time
        response = self.client.get(reverse('exordium:index'))
        self.assertEqual(response.status_code, 200)
        self.assertQuerysetEqual(response.context['album_list'].data,
            [repr(al) for al in [album_1, album_2]])
        self.assertContains(response, 'Album 1')
        self.assertContains(response, 'Album 2')
        self.assertContains(response, '"?sort=year"')

        # Now sort by year
        response = self.client.get(reverse('exordium:index'), {'sort': 'year'})
        self.assertEqual(response.status_code, 200)
        self.assertQuerysetEqual(response.context['album_list'].data,
            [repr(al) for al in [album_2, album_1]])
        self.assertContains(response, 'Album 1')
        self.assertContains(response, 'Album 2')
        self.assertContains(response, '"?sort=-year"')




from .base import ExordiumUserTests

from django.urls import reverse

from exordium.models import Artist, Album, Song, App, AlbumArt

class LibraryUpdateViewTests(ExordiumUserTests):
    """
    Testing of our library update view.  All we're really checking here
    is that the update process takes place - all the actual add/update
    functionality is tested in BasicUpdateTests.
    """

    def get_content(self, streaming_content):
        """
        Take an iterable streaming_content and return a block of text.
        """
        contents = []
        for line in streaming_content:
            contents.append(str(line))
        return "\n".join(contents)

    def test_without_permission(self):
        """
        Test when we're not actually logged in.
        """

        url = reverse('exordium:library_update')
        response = self.client.get(url)
        self.assertRedirects(response, '%s?next=%s' % (reverse('admin:login'), url))

    def test_with_permission(self):
        """
        Test when we're logged in - make sure an add actually happens.
        """

        self.longMessage = False

        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')

        self.login()
        response = self.client.get(reverse('exordium:library_update'), {'type': 'add'})
        self.assertEqual(response.status_code, 200)
        # Since LibraryAddView returns a StreamingHttpResponse, we need to
        # iterate through it for anything to actually happen.
        content = self.get_content(response.streaming_content)
        self.assertNotIn('Showing debug output', content,
            msg='Debug output found where it should not be')

        self.assertEqual(Artist.objects.count(), 2)
        self.assertEqual(Album.objects.count(), 1)
        self.assertEqual(Song.objects.count(), 1)

    def test_update_with_permission(self):
        """
        Test when we're logged in - make sure an update actually happens.
        """

        self.longMessage = False

        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.run_add()

        self.update_mp3(filename='song1.mp3', title='New Title')

        self.login()
        response = self.client.get(reverse('exordium:library_update'), {'type': 'update'})
        self.assertEqual(response.status_code, 200)
        # Since LibraryAddView returns a StreamingHttpResponse, we need to
        # iterate through it for anything to actually happen.
        content = self.get_content(response.streaming_content)
        self.assertNotIn('Showing debug output', content,
            msg='Debug output found where it should not be')

        self.assertEqual(Artist.objects.count(), 2)
        self.assertEqual(Album.objects.count(), 1)
        self.assertEqual(Song.objects.count(), 1)

        song = Song.objects.get()
        self.assertEqual(song.title, 'New Title')

    def test_no_querystring(self):
        """
        Test what happens when we don't have any querystring.  We should be
        redirected back to the library page with an error.
        """

        self.login()
        response = self.client.get(reverse('exordium:library_update'))
        self.assertRedirects(response, reverse('exordium:library'),
            fetch_redirect_response=False)

        response = self.client.get(reverse('exordium:library'))
        self.assertContains(response, 'No update type specified')

    def test_invalid_type(self):
        """
        Test what happens when we pass in an invalid update type.  Should be
        redirected back to the library page with an error.
        """

        self.login()
        response = self.client.get(reverse('exordium:library_update'), {'type': 'foo'})
        self.assertRedirects(response, reverse('exordium:library'),
            fetch_redirect_response=False)

        response = self.client.get(reverse('exordium:library'))
        self.assertContains(response, 'Invalid update type specified')

    def test_debug_checkbox_add(self):
        """
        Test the debug checkbox being active.  Should give us a note about including
        debug output.  (Though we don't really check for the actual debug output
        at the moment.)
        """

        self.longMessage = False

        self.login()
        response = self.client.get(reverse('exordium:library_update'),
            {'type': 'add', 'debug': 'yes'})
        self.assertEqual(response.status_code, 200)
        # Since LibraryAddView returns a StreamingHttpResponse, we need to
        # iterate through it for anything to actually happen.
        content = self.get_content(response.streaming_content)

        self.assertIn('Showing debug output', content, msg='Debug output not found')

    def test_debug_checkbox_update(self):
        """
        Test the debug checkbox being active.  Should give us a note about including
        debug output.  (Though we don't really check for the actual debug output
        at the moment.)
        """

        self.longMessage = False

        self.login()
        response = self.client.get(reverse('exordium:library_update'),
            {'type': 'update', 'debug': 'yes'})
        self.assertEqual(response.status_code, 200)
        # Since LibraryAddView returns a StreamingHttpResponse, we need to
        # iterate through it for anything to actually happen.
        content = self.get_content(response.streaming_content)

        self.assertIn('Showing debug output', content, msg='Debug output not found')




from .base import ExordiumUserTests

from django.urls import reverse

import os
import shutil
import zipfile
import tempfile

from exordium.models import Artist, Album, Song, App, AlbumArt

class AlbumDownloadViewTests(ExordiumUserTests):
    """
    Tests for album downloads
    """

    def setUp(self):
        """
        For these tests we need to have a place to store our zipfiles
        """
        super(AlbumDownloadViewTests, self).setUp()
        self.zipfile_path = tempfile.mkdtemp()
        self.prefs['exordium__zipfile_path'] = self.zipfile_path
        self.prefs['exordium__zipfile_url'] = 'http://testserver-zip/zipfiles'

    def tearDown(self):
        """
        Get rid of our zipfile download path
        """
        super(AlbumDownloadViewTests, self).tearDown()
        if os.path.exists(self.zipfile_path):
            shutil.rmtree(self.zipfile_path)

    def test_model_support_zipfile_no_zip_dir(self):
        """
        Tests a few conditions of our App.support_zipfile() method to ensure that
        it returns False when the zip dir doesn't exist.
        """
        # Should start out True
        self.assertEqual(App.support_zipfile(), True)
        
        # Should return False if we're configured but the zip dir doesn't
        # actually exist
        shutil.rmtree(self.zipfile_path)
        self.assertEqual(App.support_zipfile(), False)

    def test_model_support_zipfile_no_zip_path(self):
        """
        Tests a few conditions of our App.support_zipfile() method to ensure that
        it returns False when the zip path isn't configured
        """
        # Should start out True
        self.assertEqual(App.support_zipfile(), True)

        # Return false when the configured path is blank
        self.prefs['exordium__zipfile_path'] = ''
        self.assertEqual(App.support_zipfile(), False)

    def test_model_support_zipfile_no_zip_url(self):
        """
        Tests a few conditions of our App.support_zipfile() method to ensure that
        it returns False when the zip path isn't configured
        """

        # Should start out True
        self.assertEqual(App.support_zipfile(), True)

        # When the URL isn't configured, return False
        self.prefs['exordium__zipfile_url'] = ''
        self.assertEqual(App.support_zipfile(), False)

    def test_library_view_show_zipfile(self):
        """
        Test our library management view when we have zipfile configured.  Should
        show our zipfile info.  (The rest of the library management page is tested
        in ``LibraryViewTests``)
        """

        self.login()
        response = self.client.get(reverse('exordium:library'))
        self.assertEqual(response.status_code, 200)

        App.ensure_prefs()
        self.assertContains(response, 'Zipfile Support:</strong> Yes')
        self.assertNotContains(response, 'Zipfile Support:</strong> No')
        self.assertContains(response, App.prefs['exordium__zipfile_url'])
        self.assertContains(response, App.prefs['exordium__zipfile_path'])

    def test_download_button_present(self):
        """
        Test to ensure that the album view renders our download button
        now that we have the necessary preferences set.  The inverse of
        this is taken care of via various tests in ``AlbumViewTests``.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        album = Album.objects.get()

        response = self.client.get(reverse('exordium:album', args=(album.pk,)))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '"%s"' % (reverse('exordium:albumdownload', args=(album.pk,))))

    def test_album_download_without_configuration(self):
        """
        Test what happens when someone requests a download when we're
        not actually configured for it.
        """

        # Have to clear out these vars for this test
        self.prefs['exordium__zipfile_path'] = ''
        self.prefs['exordium__zipfile_url'] = ''

        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        album = Album.objects.get()

        response = self.client.get(reverse('exordium:albumdownload', args=(album.pk,)))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Exordium is not currently configured to allow zipfile creation')
        self.assertNotContains(response, '"%s"' % (reverse('exordium:albumdownload', args=(album.pk,))))
        self.assertNotContains(response, 'meta http-equiv')

    def test_model_create_zip_without_configuration(self):
        """
        Test what happens when ``Album.create_zip()`` is called when
        we're not actually configured for it.  This isn't actually possible
        in the app currently since that is checked before we even get to
        that point, but we'll test regardless.
        """

        # Have to clear out these vars for this test
        self.prefs['exordium__zipfile_path'] = ''
        self.prefs['exordium__zipfile_url'] = ''

        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        album = Album.objects.get()

        self.assertRaises(App.AlbumZipfileNotSupported, album.create_zip)

    def test_basic_album_download(self):
        """
        Test to ensure that we can generate zipfiles
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3', path='Album')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        album = Album.objects.get()

        self.assertEqual(Song.objects.count(), 1)
        song = Song.objects.get()

        response = self.client.get(reverse('exordium:albumdownload', args=(album.pk,)))
        self.assertEqual(response.status_code, 200)
        self.assertIn('filenames', response.context)
        self.assertIn('zip_file', response.context)
        self.assertIn('zip_url', response.context)
        self.assertEqual(response.context['filenames'], ['Album/song1.mp3'])
        self.assertEqual(response.context['zip_file'], 'Artist_-_Album.zip')
        self.assertContains(response, 'Album/song1.mp3<')
        self.assertContains(response, response.context['zip_file'])
        self.assertContains(response, response.context['zip_url'])
        self.assertContains(response, 'meta http-equiv')
        zip_file = os.path.join(self.zipfile_path, response.context['zip_file'])
        self.assertEqual(os.path.exists(zip_file), True)

        with zipfile.ZipFile(zip_file, 'r') as zf:
            self.assertEqual(zf.namelist(), ['Album/song1.mp3'])

    def test_basic_album_download_at_library_base(self):
        """
        Test to ensure that we can generate zipfiles from files stored
        right at the library base directory.  The top-level dir stored
        in the zipfile should be the name of the library dir itself,
        which in our case is gonna be the bit of randomness provided
        by ``tempfile.mkdtemp()``.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        album = Album.objects.get()

        self.assertEqual(Song.objects.count(), 1)
        song = Song.objects.get()

        library_base = os.path.basename(self.library_path)
        song_full = os.path.join(library_base, 'song1.mp3')

        response = self.client.get(reverse('exordium:albumdownload', args=(album.pk,)))
        self.assertEqual(response.status_code, 200)
        self.assertIn('filenames', response.context)
        self.assertIn('zip_file', response.context)
        self.assertIn('zip_url', response.context)
        self.assertEqual(response.context['filenames'], [song_full])
        self.assertEqual(response.context['zip_file'], 'Artist_-_Album.zip')
        self.assertContains(response, '%s<' % (song_full))
        self.assertContains(response, response.context['zip_file'])
        self.assertContains(response, response.context['zip_url'])
        self.assertContains(response, 'meta http-equiv')
        zip_file = os.path.join(self.zipfile_path, response.context['zip_file'])
        self.assertEqual(os.path.exists(zip_file), True)

        with zipfile.ZipFile(zip_file, 'r') as zf:
            self.assertEqual(zf.namelist(), [song_full])

    def test_basic_album_download_song_before_1980(self):
        """
        Test to ensure that we can generate zipfiles with files created
        before 1980.  Zipfiles can't store dates prior to 1980, so the
        zip process will fail.  We catch that exception in the app and
        then do some nonsense to inject the song with the current
        timestamp, so we'd like to make sure that works.  Pretty unlikely,
        but I ended up having a file right near the epoch (possibly on
        the epoch) due to some past weirdness, no doubt.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3', path='Album')
        self.run_add()
        self.set_file_ancient('Album/song1.mp3')

        self.assertEqual(Album.objects.count(), 1)
        album = Album.objects.get()

        self.assertEqual(Song.objects.count(), 1)
        song = Song.objects.get()

        response = self.client.get(reverse('exordium:albumdownload', args=(album.pk,)))
        self.assertEqual(response.status_code, 200)
        self.assertIn('filenames', response.context)
        self.assertIn('zip_file', response.context)
        self.assertIn('zip_url', response.context)
        self.assertEqual(response.context['filenames'], ['Album/song1.mp3'])
        self.assertEqual(response.context['zip_file'], 'Artist_-_Album.zip')
        self.assertContains(response, 'Album/song1.mp3<')
        self.assertContains(response, response.context['zip_file'])
        self.assertContains(response, response.context['zip_url'])
        self.assertContains(response, 'meta http-equiv')
        zip_file = os.path.join(self.zipfile_path, response.context['zip_file'])
        self.assertEqual(os.path.exists(zip_file), True)

        with zipfile.ZipFile(zip_file, 'r') as zf:
            self.assertEqual(zf.namelist(), ['Album/song1.mp3'])

    def test_basic_album_download_twice(self):
        """
        Generate a zipfile and then request the zipfile again.  Should get a page
        back which has a link to the zipfile, and a message about how it was
        generated previously.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3', path='Album')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        album = Album.objects.get()

        self.assertEqual(Song.objects.count(), 1)
        song = Song.objects.get()

        # Original request
        response = self.client.get(reverse('exordium:albumdownload', args=(album.pk,)))
        self.assertEqual(response.status_code, 200)

        # Subsequent request
        response = self.client.get(reverse('exordium:albumdownload', args=(album.pk,)))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Zipfile already exists.  You should be able to download with the link below.')
        self.assertNotIn('filenames', response.context)
        self.assertIn('zip_file', response.context)
        self.assertIn('zip_url', response.context)
        self.assertEqual(response.context['zip_file'], 'Artist_-_Album.zip')
        self.assertNotContains(response, 'Album/song1.mp3<')
        self.assertContains(response, response.context['zip_file'])
        self.assertContains(response, response.context['zip_url'])
        self.assertContains(response, 'meta http-equiv')
        zip_file = os.path.join(self.zipfile_path, response.context['zip_file'])
        self.assertEqual(os.path.exists(zip_file), True)

        with zipfile.ZipFile(zip_file, 'r') as zf:
            self.assertEqual(zf.namelist(), ['Album/song1.mp3'])

    def test_basic_album_download_with_art(self):
        """
        Test to ensure that we can generate zipfiles, with album art included.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3', path='Album')
        self.add_art(path='Album')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        album = Album.objects.get()

        self.assertEqual(Song.objects.count(), 1)
        song = Song.objects.get()

        response = self.client.get(reverse('exordium:albumdownload', args=(album.pk,)))
        self.assertEqual(response.status_code, 200)
        self.assertIn('filenames', response.context)
        self.assertIn('zip_file', response.context)
        self.assertIn('zip_url', response.context)
        self.assertEqual(response.context['filenames'], ['Album/song1.mp3', 'Album/cover.jpg'])
        self.assertEqual(response.context['zip_file'], 'Artist_-_Album.zip')
        self.assertContains(response, 'Album/song1.mp3<')
        self.assertContains(response, 'Album/cover.jpg<')
        self.assertContains(response, response.context['zip_file'])
        self.assertContains(response, response.context['zip_url'])
        self.assertContains(response, 'meta http-equiv')
        zip_file = os.path.join(self.zipfile_path, response.context['zip_file'])
        self.assertEqual(os.path.exists(zip_file), True)

        with zipfile.ZipFile(zip_file, 'r') as zf:
            self.assertEqual(zf.namelist(), ['Album/song1.mp3', 'Album/cover.jpg'])

    def test_basic_album_download_with_art_in_parent_dir(self):
        """
        Test to ensure that we can generate zipfiles, with album art included
        but in the parent directory.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3', path='Artist/Album')
        self.add_art(path='Artist')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        album = Album.objects.get()

        self.assertEqual(Song.objects.count(), 1)
        song = Song.objects.get()

        response = self.client.get(reverse('exordium:albumdownload', args=(album.pk,)))
        self.assertEqual(response.status_code, 200)
        self.assertIn('filenames', response.context)
        self.assertIn('zip_file', response.context)
        self.assertIn('zip_url', response.context)
        self.assertEqual(response.context['filenames'], ['Artist/Album/song1.mp3', 'Artist/cover.jpg'])
        self.assertEqual(response.context['zip_file'], 'Artist_-_Album.zip')
        self.assertContains(response, 'Artist/Album/song1.mp3<')
        self.assertContains(response, 'Artist/cover.jpg<')
        self.assertContains(response, response.context['zip_file'])
        self.assertContains(response, response.context['zip_url'])
        self.assertContains(response, 'meta http-equiv')
        zip_file = os.path.join(self.zipfile_path, response.context['zip_file'])
        self.assertEqual(os.path.exists(zip_file), True)

        with zipfile.ZipFile(zip_file, 'r') as zf:
            self.assertEqual(zf.namelist(), ['Artist/Album/song1.mp3', 'Artist/cover.jpg'])

    def test_miscellaneous_album_download_in_multiple_dirs(self):
        """
        Test to ensure that we can generate zipfiles for an album which is contained
        in more than one directory.  (Using a 'miscellaneous' album here but this'd
        work for any album which happens to do this)
        """
        self.add_mp3(artist='Artist', title='Title 1',
            filename='song1.mp3', path='Artist/Tracks')
        self.add_mp3(artist='Artist', title='Title 2',
            filename='song2.mp3', path='Artist/MoreTracks')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        album = Album.objects.get()

        self.assertEqual(Song.objects.count(), 2)

        response = self.client.get(reverse('exordium:albumdownload', args=(album.pk,)))
        self.assertEqual(response.status_code, 200)
        self.assertIn('filenames', response.context)
        self.assertIn('zip_file', response.context)
        self.assertIn('zip_url', response.context)
        self.assertEqual(
            sorted(response.context['filenames']),
            sorted(['Artist/Tracks/song1.mp3', 'Artist/MoreTracks/song2.mp3'])
        )
        self.assertEqual(response.context['zip_file'], 'Artist_-_%s.zip' % (App.norm_filename(album.name)))
        self.assertContains(response, 'Artist/Tracks/song1.mp3<')
        self.assertContains(response, 'Artist/MoreTracks/song2.mp3<')
        self.assertContains(response, response.context['zip_file'])
        self.assertContains(response, response.context['zip_url'])
        self.assertContains(response, 'meta http-equiv')
        zip_file = os.path.join(self.zipfile_path, response.context['zip_file'])
        self.assertEqual(os.path.exists(zip_file), True)

        with zipfile.ZipFile(zip_file, 'r') as zf:
            self.assertEqual(
                sorted(zf.namelist()),
                sorted(['Artist/Tracks/song1.mp3', 'Artist/MoreTracks/song2.mp3'])
            )




from .base import ExordiumUserTests

from django.urls import reverse

from django.templatetags.static import static

from exordium.models import Artist, Album, Song, App, AlbumArt

class AlbumViewTests(ExordiumUserTests):
    """
    Tests of our Album info page
    """

    def test_invalid_album(self):
        """
        Tests making a request for an album which can't be found.
        """
        response = self.client.get(reverse('exordium:album', args=(42,)))
        self.assertEqual(response.status_code, 404)

    def test_minimal_album(self):
        """
        Test a minimally-tagged album
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        album = Album.objects.get()

        self.assertEqual(Song.objects.count(), 1)
        song = Song.objects.get()

        response = self.client.get(reverse('exordium:album', args=(album.pk,)))
        self.assertEqual(response.status_code, 200)
        self.assertQuerysetEqual(response.context['songs'].data, [repr(song)])
        self.assertEqual(response.context['groups'], [])
        self.assertEqual(response.context['composers'], [])
        self.assertEqual(response.context['conductors'], [])
        self.assertNotContains(response, 'Ensemble')
        self.assertNotContains(response, 'Conductor')
        self.assertNotContains(response, 'Composer')
        self.assertContains(response, reverse('exordium:artist', args=(album.artist.normname,)))
        self.assertContains(response, str(album))
        self.assertContains(response, str(album.artist))
        self.assertNotContains(response, 'Year:')
        self.assertContains(response, 'Tracks: <strong>1</strong>')
        self.assertContains(response, 'Length: <strong>0:02</strong>')
        self.assertContains(response, 'Added on:')
        self.assertContains(response, reverse('exordium:m3udownload', args=(album.pk,)))
        self.assertContains(response, 'albumstreambutton')
        self.assertContains(response, '"%s"' % (static('exordium/no_album_art.png')))
        self.assertContains(response, song.title)
        self.assertContains(response, '1 item')

        # Ensure we have a tracknum column, but not an album column
        self.assertContains(response, '"?sort=tracknum"')
        self.assertNotContains(response, '"?sort=album"')

        # At the moment we do not have album downloads enabled, so we should not see
        # the download button.
        self.assertNotContains(response, reverse('exordium:albumdownload', args=(album.pk,)))

        # We are not logged in, so we shouldn't see the album art regen button
        self.assertNotContains(response, reverse('exordium:albumartupdate', args=(album.pk,)))

    def test_minimal_album_art(self):
        """
        Test a minimally-tagged album which also has album art.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art()
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        album = Album.objects.get()

        self.assertEqual(Song.objects.count(), 1)
        song = Song.objects.get()

        response = self.client.get(reverse('exordium:album', args=(album.pk,)))
        self.assertEqual(response.status_code, 200)
        self.assertQuerysetEqual(response.context['songs'].data, [repr(song)])
        self.assertEqual(response.context['groups'], [])
        self.assertEqual(response.context['composers'], [])
        self.assertEqual(response.context['conductors'], [])
        self.assertNotContains(response, 'Ensemble')
        self.assertNotContains(response, 'Conductor')
        self.assertNotContains(response, 'Composer')
        self.assertContains(response, reverse('exordium:artist', args=(album.artist.normname,)))
        self.assertContains(response, str(album))
        self.assertContains(response, str(album.artist))
        self.assertNotContains(response, 'Year:')
        self.assertContains(response, 'Tracks: <strong>1</strong>')
        self.assertContains(response, 'Length: <strong>0:02</strong>')
        self.assertContains(response, 'Added on:')
        self.assertContains(response, reverse('exordium:m3udownload', args=(album.pk,)))
        self.assertContains(response, 'albumstreambutton')
        self.assertNotContains(response, '"%s"' % (static('exordium/no_album_art.png')))
        self.assertContains(response, reverse('exordium:albumart', args=(album.pk, 'album',)))
        self.assertContains(response, reverse('exordium:origalbumart', args=(album.pk, album.art_ext,)))
        self.assertContains(response, song.title)
        self.assertContains(response, '1 item')

        # Ensure we have a tracknum column, but not an album column
        self.assertContains(response, '"?sort=tracknum"')
        self.assertNotContains(response, '"?sort=album"')

        # At the moment we do not have album downloads enabled, so we should not see
        # the download button.
        self.assertNotContains(response, reverse('exordium:albumdownload', args=(album.pk,)))

        # We are not logged in, so we shouldn't see the album art regen button
        self.assertNotContains(response, reverse('exordium:albumartupdate', args=(album.pk,)))

    def test_login_album_no_art(self):
        """
        Test view when we're logged in and have no album art.  Should see our
        update button now.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        album = Album.objects.get()

        self.login()
        response = self.client.get(reverse('exordium:album', args=(album.pk,)))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '"%s"' % (static('exordium/no_album_art.png')))
        self.assertContains(response, reverse('exordium:albumartupdate', args=(album.pk,)))

    def test_login_album_with_art(self):
        """
        Test view when we're logged in and have album art.  Should have our update button.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art()
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        album = Album.objects.get()

        self.login()
        response = self.client.get(reverse('exordium:album', args=(album.pk,)))
        self.assertEqual(response.status_code, 200)
        self.assertNotContains(response, '"%s"' % (static('exordium/no_album_art.png')))
        self.assertContains(response, reverse('exordium:albumart', args=(album.pk, 'album',)))
        self.assertContains(response, reverse('exordium:origalbumart', args=(album.pk, album.art_ext,)))
        self.assertContains(response, reverse('exordium:albumartupdate', args=(album.pk,)))

    def test_fully_tagged_album(self):
        """
        Test a fully-tagged album
        """
        self.add_mp3(artist='Artist', title='Title 1', tracknum=1,
            year=2016, group='Group', conductor='Conductor',
            composer='Composer',
            album='Album', filename='song1.mp3')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        album = Album.objects.get()

        self.assertEqual(Song.objects.count(), 1)
        song = Song.objects.get()

        artist = Artist.objects.get(name='Artist')
        group = Artist.objects.get(name='Group')
        conductor = Artist.objects.get(name='Conductor')
        composer = Artist.objects.get(name='Composer')

        response = self.client.get(reverse('exordium:album', args=(album.pk,)))
        self.assertEqual(response.status_code, 200)
        self.assertQuerysetEqual(response.context['songs'].data, [repr(song)])
        self.assertQuerysetEqual(response.context['groups'], [repr(group)])
        self.assertQuerysetEqual(response.context['composers'], [repr(composer)])
        self.assertQuerysetEqual(response.context['conductors'], [repr(conductor)])
        self.assertEqual(response.context['have_empty_group'], False)
        self.assertEqual(response.context['have_empty_composer'], False)
        self.assertEqual(response.context['have_empty_conductor'], False)
        self.assertContains(response, 'Ensemble:')
        self.assertContains(response, 'Conductor:')
        self.assertContains(response, 'Composer:')
        self.assertContains(response, reverse('exordium:artist', args=(album.artist.normname,)))
        self.assertContains(response, reverse('exordium:artist', args=(group.normname,)))
        self.assertContains(response, reverse('exordium:artist', args=(conductor.normname,)))
        self.assertContains(response, reverse('exordium:artist', args=(composer.normname,)))
        self.assertContains(response, str(album))
        self.assertContains(response, str(album.artist))
        self.assertContains(response, str(group))
        self.assertContains(response, str(conductor))
        self.assertContains(response, str(composer))
        self.assertContains(response, 'Year: <strong>2016</strong>')
        self.assertContains(response, 'Tracks: <strong>1</strong>')
        self.assertContains(response, 'Length: <strong>0:02</strong>')
        self.assertContains(response, song.title)
        self.assertContains(response, '"?sort=tracknum"')
        self.assertContains(response, '1 item')

    def test_fully_tagged_album_two_tracks(self):
        """
        Test a fully-tagged album, with two tracks
        """
        self.add_mp3(artist='Artist', title='Title 1', tracknum=1,
            year=2016, group='Group', conductor='Conductor',
            composer='Composer',
            album='Album', filename='song1.mp3')
        self.add_mp3(artist='Artist', title='Title 2', tracknum=2,
            year=2016, group='Group 2', conductor='Conductor 2',
            composer='Composer 2',
            album='Album', filename='song2.mp3')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        album = Album.objects.get()

        self.assertEqual(Song.objects.count(), 2)
        songs = [
            Song.objects.get(filename='song1.mp3'),
            Song.objects.get(filename='song2.mp3'),
        ]

        artist = Artist.objects.get(name='Artist')
        groups = [
            Artist.objects.get(name='Group'),
            Artist.objects.get(name='Group 2'),
        ]
        conductors = [
            Artist.objects.get(name='Conductor'),
            Artist.objects.get(name='Conductor 2'),
        ]
        composers = [
            Artist.objects.get(name='Composer'),
            Artist.objects.get(name='Composer 2'),
        ]

        response = self.client.get(reverse('exordium:album', args=(album.pk,)))
        self.assertEqual(response.status_code, 200)
        self.assertQuerysetEqual(response.context['songs'].data, [repr(song) for song in songs])
        self.assertQuerysetEqual(response.context['groups'], [repr(group) for group in groups])
        self.assertQuerysetEqual(response.context['composers'], [repr(composer) for composer in composers])
        self.assertQuerysetEqual(response.context['conductors'], [repr(conductor) for conductor in conductors])
        self.assertEqual(response.context['have_empty_group'], False)
        self.assertEqual(response.context['have_empty_composer'], False)
        self.assertEqual(response.context['have_empty_conductor'], False)
        self.assertContains(response, 'Ensembles:')
        self.assertContains(response, 'Conductors:')
        self.assertContains(response, 'Composers:')
        for a in [artist] + groups + conductors + composers:
            self.assertContains(response, reverse('exordium:artist', args=(a.normname,)))
            self.assertContains(response, str(a))
        self.assertContains(response, str(album))
        self.assertContains(response, str(album.artist))
        self.assertContains(response, 'Year: <strong>2016</strong>')
        self.assertContains(response, 'Tracks: <strong>2</strong>')
        self.assertContains(response, 'Length: <strong>0:04</strong>')
        for song in songs:
            self.assertContains(response, song.title)
        self.assertContains(response, '"?sort=tracknum"')
        self.assertContains(response, '2 items')

    def test_fully_tagged_album_two_tracks_various(self):
        """
        Test a fully-tagged album, with two tracks, which is also a various-artists
        album.
        """
        self.add_mp3(artist='Artist 1', title='Title 1', tracknum=1,
            year=2016, group='Group 1', conductor='Conductor 1',
            composer='Composer 1',
            album='Album', filename='song1.mp3')
        self.add_mp3(artist='Artist 2', title='Title 2', tracknum=2,
            year=2016, group='Group 2', conductor='Conductor 2',
            composer='Composer 2',
            album='Album', filename='song2.mp3')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        album = Album.objects.get()

        self.assertEqual(Song.objects.count(), 2)
        songs = [
            Song.objects.get(filename='song1.mp3'),
            Song.objects.get(filename='song2.mp3'),
        ]

        various = Artist.objects.get(name='Various')
        artists = [
            Artist.objects.get(name='Artist 1'),
            Artist.objects.get(name='Artist 2'),
        ]
        groups = [
            Artist.objects.get(name='Group 1'),
            Artist.objects.get(name='Group 2'),
        ]
        conductors = [
            Artist.objects.get(name='Conductor 1'),
            Artist.objects.get(name='Conductor 2'),
        ]
        composers = [
            Artist.objects.get(name='Composer 1'),
            Artist.objects.get(name='Composer 2'),
        ]

        response = self.client.get(reverse('exordium:album', args=(album.pk,)))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Various')
        self.assertContains(response, reverse('exordium:artist', args=(various.normname,)))
        self.assertQuerysetEqual(response.context['songs'].data, [repr(song) for song in songs])
        self.assertQuerysetEqual(response.context['groups'], [repr(group) for group in groups])
        self.assertQuerysetEqual(response.context['composers'], [repr(composer) for composer in composers])
        self.assertQuerysetEqual(response.context['conductors'], [repr(conductor) for conductor in conductors])
        self.assertEqual(response.context['have_empty_group'], False)
        self.assertEqual(response.context['have_empty_composer'], False)
        self.assertEqual(response.context['have_empty_conductor'], False)
        self.assertContains(response, 'Ensembles:')
        self.assertContains(response, 'Conductors:')
        self.assertContains(response, 'Composers:')
        for a in artists + groups + conductors + composers:
            self.assertContains(response, reverse('exordium:artist', args=(a.normname,)))
            self.assertContains(response, str(a))
        self.assertContains(response, str(album))
        self.assertContains(response, str(album.artist))
        self.assertContains(response, 'Year: <strong>2016</strong>')
        self.assertContains(response, 'Tracks: <strong>2</strong>')
        self.assertContains(response, 'Length: <strong>0:04</strong>')
        for song in songs:
            self.assertContains(response, song.title)
        self.assertContains(response, '"?sort=tracknum"')
        self.assertContains(response, '2 items')

    def test_album_some_tracks_with_classical_tags_others_without(self):
        """
        Tests display of an album in which one track has classical tags defined
        but the other does not.  The classical tags should be displayed up in
        the top header, but also with a note that some tracks don't have the
        tags.  Technically we should have a check for making sure the tracks
        themselves are showing the information, though we can't really do that
        without hooking into selenium or whatever.
        """
        self.add_mp3(artist='Artist', title='Title 1', tracknum=1,
            year=2016, group='Group', conductor='Conductor',
            composer='Composer',
            album='Album', filename='song1.mp3')
        self.add_mp3(artist='Artist', title='Title 2', tracknum=2,
            year=2016, album='Album', filename='song2.mp3')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        album = Album.objects.get()

        self.assertEqual(Song.objects.count(), 2)
        songs = [
            Song.objects.get(filename='song1.mp3'),
            Song.objects.get(filename='song2.mp3'),
        ]

        artist = Artist.objects.get(name='Artist')
        group = Artist.objects.get(name='Group')
        conductor = Artist.objects.get(name='Conductor')
        composer = Artist.objects.get(name='Composer')

        response = self.client.get(reverse('exordium:album', args=(album.pk,)))
        self.assertEqual(response.status_code, 200)
        self.assertQuerysetEqual(response.context['songs'].data, [repr(song) for song in songs])
        self.assertQuerysetEqual(response.context['groups'], [repr(group)])
        self.assertQuerysetEqual(response.context['composers'], [repr(composer)])
        self.assertQuerysetEqual(response.context['conductors'], [repr(conductor)])
        self.assertEqual(response.context['have_empty_group'], True)
        self.assertEqual(response.context['have_empty_composer'], True)
        self.assertEqual(response.context['have_empty_conductor'], True)
        self.assertContains(response, 'Ensemble:')
        self.assertContains(response, 'Conductor:')
        self.assertContains(response, 'Composer:')
        self.assertContains(response, 'Some tracks have no ensemble')
        self.assertContains(response, 'Some tracks have no conductor')
        self.assertContains(response, 'Some tracks have no composer')
        for a in [artist, group, conductor, composer]:
            self.assertContains(response, reverse('exordium:artist', args=(a.normname,)))
            self.assertContains(response, str(a))
        self.assertContains(response, str(album))
        self.assertContains(response, str(album.artist))
        self.assertContains(response, 'Year: <strong>2016</strong>')
        self.assertContains(response, 'Tracks: <strong>2</strong>')
        self.assertContains(response, 'Length: <strong>0:04</strong>')
        for song in songs:
            self.assertContains(response, song.title)
        self.assertContains(response, '"?sort=tracknum"')
        self.assertContains(response, '2 items')

    def test_miscellaneous_album(self):
        """
        Test a miscellaneous (non-album-tracks) album.  The only real difference
        is that this table should NOT contain a tracknum column.
        """
        self.add_mp3(artist='Artist', title='Title 1', filename='song1.mp3')
        self.run_add()

        self.assertEqual(Song.objects.count(), 1)
        song = Song.objects.get()

        self.assertEqual(Album.objects.count(), 1)
        album = Album.objects.get()

        response = self.client.get(reverse('exordium:album', args=(album.pk,)))
        self.assertEqual(response.status_code, 200)
        self.assertQuerysetEqual(response.context['songs'].data, [repr(song)])
        self.assertContains(response, song.title)
        self.assertNotContains(response, '"?sort=tracknum"')
        self.assertContains(response, '1 item')

    def test_sorting_song(self):
        """
        Test at least one case of song sorting.  Default sorting is by track
        number.
        """
        self.add_mp3(artist='Artist', title='Title 3', tracknum=1,
            album='Album', filename='song1.mp3')
        self.add_mp3(artist='Artist', title='Title 2', tracknum=2,
            album='Album', filename='song2.mp3')
        self.add_mp3(artist='Artist', title='Title 1', tracknum=3,
            album='Album', filename='song3.mp3')
        self.run_add()

        songs = [
            Song.objects.get(filename='song1.mp3'),
            Song.objects.get(filename='song2.mp3'),
            Song.objects.get(filename='song3.mp3'),
        ]
        album = Album.objects.get(name='Album')

        response = self.client.get(reverse('exordium:album', args=(album.pk,)))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['songs'].data), 3)
        self.assertQuerysetEqual(response.context['songs'].data, [repr(song) for song in songs])
        self.assertContains(response, '"?sort=title"')
        self.assertContains(response, '3 items')

        # test the sorting button
        response = self.client.get(reverse('exordium:album', args=(album.pk,)), {'sort': 'title'})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['songs'].data), 3)
        self.assertQuerysetEqual(response.context['songs'].data, [repr(song) for song in reversed(songs)])
        self.assertContains(response, '"?sort=-title"')
        self.assertContains(response, '3 items')

    def test_pagination(self):
        """
        Test pagination.  Our album view will show 100 tracks, so rather than
        going through our whole ``run_add()`` process, we're just importing
        directly into the database.  Should probably do this for the rest of our
        pagination tests, too, actually, given that ``run_add()`` can be rather
        slow.
        """
        artist = Artist.objects.create(name='Artist', normname='artist')
        album = Album.objects.create(artist=artist, name='Album', normname='album')
        songs = {}
        for num in range(120):
            songs[num] = Song.objects.create(filename='file%03d.mp3' % (num+1),
                artist=artist,
                album=album,
                title='Title %03d' % (num+1),
                year=2016,
                tracknum=num+1,
                normtitle='title %03d' % (num+1),
                raw_artist='artist',
                filetype=Song.MP3,
                bitrate=128000,
                mode=Song.CBR,
                size=123000,
                length=90,
                sha256sum='0cf31fc7d968ec16c69758f9b0ebb2355471d5694a151b40e5e4f8641b061092',
            )

        response = self.client.get(reverse('exordium:album', args=(album.pk,)))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '100 of 120 items')
        self.assertContains(response, '"?page=2"')
        self.assertEqual(len(response.context['songs'].data), 120)
        for num in range(100):
            self.assertContains(response, '%s<' % (songs[num]))
            self.assertContains(response, "'%s'" % (songs[num].get_download_url_html5()))
            self.assertContains(response, '"%s"' % (songs[num].get_download_url_m3u()))
        for num in range(100, 120):
            self.assertNotContains(response, '%s<' % (songs[num]))
            # Note that our album-streaming button *will* have all html5 results in there,
            # even stuff from future pages
            self.assertContains(response, "'%s'" % (songs[num].get_download_url_html5()))
            self.assertNotContains(response, '"%s"' % (songs[num].get_download_url_m3u()))

        # test page 2
        response = self.client.get(reverse('exordium:album', args=(album.pk,)), {'page': 2})
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '20 of 120 items')
        self.assertContains(response, '"?page=1"')
        self.assertEqual(len(response.context['songs'].data), 120)
        for num in range(100):
            self.assertNotContains(response, '%s<' % (songs[num]))
            # Likewise -- the album-streaming button will have everything
            self.assertContains(response, "'%s'" % (songs[num].get_download_url_html5()))
            self.assertNotContains(response, '"%s"' % (songs[num].get_download_url_m3u()))
        for num in range(100, 120):
            self.assertContains(response, '%s<' % (songs[num]))
            self.assertContains(response, "'%s'" % (songs[num].get_download_url_html5()))
            self.assertContains(response, '"%s"' % (songs[num].get_download_url_m3u()))

    def test_play_button_single(self):
        """
        Test to make sure we have a "play" button for a single (streamable) track
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        album = Album.objects.get()

        self.assertEqual(Song.objects.count(), 1)
        song = Song.objects.get()

        response = self.client.get(reverse('exordium:album', args=(album.pk,)))
        self.assertEqual(response.status_code, 200)
        self.assertQuerysetEqual(response.context['songs'].data, [repr(song)])
        self.assertContains(response, 'playbutton')
        self.assertContains(response, 'Stream this track')

    def test_no_play_button_single(self):
        """
        Test to make sure we do NOT have a "play" button for a single
        (non-streamable) track.  That just means Ogg Opus, for now.
        """
        self.add_opus(artist='Artist', title='Title 1',
            album='Album', filename='song1.opus')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        album = Album.objects.get()

        self.assertEqual(Song.objects.count(), 1)
        song = Song.objects.get()

        response = self.client.get(reverse('exordium:album', args=(album.pk,)))
        self.assertEqual(response.status_code, 200)
        self.assertQuerysetEqual(response.context['songs'].data, [repr(song)])
        self.assertNotContains(response, 'playbutton')
        self.assertNotContains(response, 'Stream this track')
        self.assertContains(response, 'Track cannot be streamed')

    def test_play_button_two_tracks_mixed(self):
        """
        Test to make sure we have both a "play" button and a non-streamable notice,
        for an album with one streamable and one non-streamable track.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_opus(artist='Artist', title='Title 2',
            album='Album', filename='song2.opus')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        album = Album.objects.get()

        self.assertEqual(Song.objects.count(), 2)
        songs = [
            Song.objects.get(filename='song1.mp3'),
            Song.objects.get(filename='song2.opus'),
        ]

        response = self.client.get(reverse('exordium:album', args=(album.pk,)))
        self.assertEqual(response.status_code, 200)
        self.assertQuerysetEqual(response.context['songs'].data, [repr(song) for song in songs])
        self.assertContains(response, 'playbutton')
        self.assertContains(response, 'Stream this track')
        self.assertContains(response, 'Track cannot be streamed')

    def test_html5_album_stream_button(self):
        """
        Test to make sure we have a jPlayer-based album stream button
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        self.assertEqual(Song.objects.count(), 1)
        album = Album.objects.get()

        response = self.client.get(reverse('exordium:album', args=(album.pk,)))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'albumstreambutton')
        self.assertNotContains(response, 'albumstreambutton" disabled')
        self.assertContains(response, '>Stream Album (HTML5 pop-up)<')

    def test_html5_no_album_stream_button(self):
        """
        Test to make sure we do NOT have a jPlayer-based album stream button,
        when there are no jPlayer-streamable tracks present
        """
        self.add_opus(artist='Artist', title='Title 1',
            album='Album', filename='song1.opus')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        self.assertEqual(Song.objects.count(), 1)
        album = Album.objects.get()

        response = self.client.get(reverse('exordium:album', args=(album.pk,)))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'albumstreambutton" disabled')
        self.assertContains(response, '>Stream Album (HTML5 pop-up - unavailable)<')

    def test_html5_album_stream_button_mixed(self):
        """
        Test to make sure we have a jPlayer-based album stream button when we
        have a mixed-jPlayer-streamable album present.  Note that this doesn't
        *really* test this fully, since we're not inspecting the contents of the
        button's Javascript, but whatever.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_opus(artist='Artist', title='Title 2',
            album='Album', filename='song2.opus')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        self.assertEqual(Song.objects.count(), 2)
        album = Album.objects.get()

        response = self.client.get(reverse('exordium:album', args=(album.pk,)))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'albumstreambutton')
        self.assertNotContains(response, 'albumstreambutton" disabled')
        self.assertContains(response, '>Stream Album (HTML5 pop-up)<')




from .base import ExordiumTests

from django.urls import reverse

from django.templatetags.static import static

from exordium.models import Artist, Album, Song, App, AlbumArt

class ArtistViewTests(ExordiumTests):
    """
    Tests of our Artist info page
    """

    def test_invalid_artist(self):
        """
        Tests making a request for an artist which can't be found.
        """
        response = self.client.get(reverse('exordium:artist', args=('notfound',)))
        self.assertEqual(response.status_code, 404)

    def test_single_album(self):
        """
        Test an artist who only has a single album
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.run_add()
        
        self.assertEqual(Artist.objects.count(), 2)
        self.assertEqual(Album.objects.count(), 1)
        self.assertEqual(Song.objects.count(), 1)

        artist = Artist.objects.get(name='Artist')
        album = Album.objects.get()
        song = Song.objects.get()

        response = self.client.get(reverse('exordium:artist', args=('artist',)))
        self.assertEqual(response.status_code, 200)
        self.assertQuerysetEqual(response.context['albums'].data, [repr(album)])
        self.assertEqual(response.context['have_songs'], True)
        self.assertQuerysetEqual(response.context['songs'].data, [repr(song)])
        self.assertContains(response, 'Songs by %s' % (artist))
        self.assertContains(response, reverse('exordium:artist', args=(artist.normname,)))
        self.assertContains(response, reverse('exordium:album', args=(album.pk,)))
        self.assertContains(response, song.get_download_url_html5())
        self.assertContains(response, song.get_download_url_m3u())
        self.assertContains(response, '1 album')
        self.assertContains(response, '1 song')

        # May as well double-check the no-album-art-found image as well
        self.assertContains(response, '"%s"' % (static('exordium/no_album_art_small.png')))

        # Artist song-list view should not have the tracknum column, and should have album-sort
        self.assertNotContains(response, 'sort=tracknum')
        self.assertContains(response, 'song-sort=album')

    def test_single_album_with_art(self):
        """
        Test an artist who only has a single album, with album art.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art()
        self.run_add()
        
        self.assertEqual(Artist.objects.count(), 2)
        self.assertEqual(Album.objects.count(), 1)
        self.assertEqual(Song.objects.count(), 1)

        artist = Artist.objects.get(name='Artist')
        album = Album.objects.get()
        song = Song.objects.get()

        response = self.client.get(reverse('exordium:artist', args=('artist',)))
        self.assertEqual(response.status_code, 200)
        self.assertQuerysetEqual(response.context['albums'].data, [repr(album)])
        self.assertEqual(response.context['have_songs'], True)
        self.assertQuerysetEqual(response.context['songs'].data, [repr(song)])
        self.assertContains(response, '1 album')
        self.assertContains(response, '1 song')
        self.assertContains(response, 'Songs by %s' % (artist))
        self.assertContains(response, reverse('exordium:artist', args=(artist.normname,)))
        self.assertContains(response, reverse('exordium:album', args=(album.pk,)))
        self.assertContains(response, reverse('exordium:albumart', args=(album.pk, 'list',)))
        # These will show up once for HTML5, and once for direct download
        self.assertContains(response, song.get_download_url_html5())
        self.assertContains(response, song.get_download_url_m3u())
        self.assertNotContains(response, 'sort=tracknum')
        self.assertContains(response, 'song-sort=album')

    def test_various_artists(self):
        """
        Test a various artist album - the one containing the artist
        we care about should show up in the list
        """
        self.add_mp3(artist='Artist 1', title='Title 1',
            album='Album 1', filename='song1.mp3', path='album_1')
        self.add_mp3(artist='Artist 2', title='Title 2',
            album='Album 1', filename='song2.mp3', path='album_1')
        self.add_mp3(artist='Artist 2', title='Title 3',
            album='Album 2', filename='song3.mp3', path='album_2')
        self.add_mp3(artist='Artist 3', title='Title 4',
            album='Album 2', filename='song4.mp3', path='album_2')
        self.run_add()

        self.assertEqual(Artist.objects.count(), 4)
        self.assertEqual(Album.objects.count(), 2)
        self.assertEqual(Song.objects.count(), 4)

        artist_1 = Artist.objects.get(name='Artist 1')
        artist_2 = Artist.objects.get(name='Artist 2')
        artist_3 = Artist.objects.get(name='Artist 3')
        various = Artist.objects.get(name='Various')

        album_1 = Album.objects.get(name='Album 1')
        album_2 = Album.objects.get(name='Album 2')

        song_1 = Song.objects.get(filename='album_1/song1.mp3')
        song_2 = Song.objects.get(filename='album_1/song2.mp3')
        song_3 = Song.objects.get(filename='album_2/song3.mp3')
        song_4 = Song.objects.get(filename='album_2/song4.mp3')

        response = self.client.get(reverse('exordium:artist', args=(artist_1.normname,)))
        self.assertEqual(response.status_code, 200)

        # Only the Various Artists album should show up
        self.assertContains(response, '1 album')
        self.assertQuerysetEqual(response.context['albums'].data, [repr(album_1)])
        self.assertContains(response, reverse('exordium:album', args=(album_1.pk,)))
        self.assertContains(response, reverse('exordium:artist', args=(various.normname,)))
        self.assertNotContains(response, str(album_2))
        self.assertNotContains(response, reverse('exordium:album', args=(album_2.pk,)))

        # Only one song should show up
        self.assertContains(response, '1 song')
        self.assertQuerysetEqual(response.context['songs'].data, [repr(song_1)])
        self.assertContains(response, reverse('exordium:artist', args=(artist_1.normname,)))
        # This check is a bit silly since it's already shown up above
        self.assertContains(response, reverse('exordium:album', args=(album_1.pk,)))
        self.assertContains(response, song_1.get_download_url_html5())
        self.assertContains(response, song_1.get_download_url_m3u())
        for song in [song_2, song_3, song_4]:
            self.assertNotContains(response, str(song))
            self.assertNotContains(response, song.get_download_url_html5())
            self.assertNotContains(response, song.get_download_url_m3u())

        # Shouldn't see any links to our other two artists
        for artist in [artist_2, artist_3]:
            self.assertNotContains(response, str(artist))
            self.assertNotContains(response, reverse('exordium:artist', args=(artist.normname,)))

    def test_ordering(self):
        """
        Tests ordering of various types of albums.  Regular albums should be first,
        followed by non-album tracks, and then Various-artists albums.  Song lists
        should be alphabetical.
        """
        # "Album 1" and "Album 3" are regular albums
        # "Album 2" is V/A
        # The remaining track will create a non-album track.
        self.add_mp3(artist='Artist 1', title='Title 1',
            album='Album 1', filename='song1.mp3', path='album_1')
        self.add_mp3(artist='Artist 1', title='Title 2',
            album='Album 1', filename='song2.mp3', path='album_1')
        self.add_mp3(artist='Artist 1', title='Title 3',
            album='Album 2', filename='song3.mp3', path='album_2')
        self.add_mp3(artist='Artist 2', title='Title 4',
            album='Album 2', filename='song4.mp3', path='album_2')
        self.add_mp3(artist='Artist 1', title='Title 5',
            album='Album 3', filename='song5.mp3', path='album_3')
        self.add_mp3(artist='Artist 1', title='Title 6',
            album='Album 3', filename='song6.mp3', path='album_3')
        self.add_mp3(artist='Artist 1', title='Title 7',
            filename='song7.mp3')
        self.run_add()

        artist = Artist.objects.get(name='Artist 1')

        self.assertEqual(Album.objects.count(), 4)
        reg_album_1 = Album.objects.get(name='Album 1')
        reg_album_2 = Album.objects.get(name='Album 3')
        va_album = Album.objects.get(name='Album 2')
        misc_album = Album.objects.get(miscellaneous=True)

        response = self.client.get(reverse('exordium:artist', args=(artist.normname,)))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '4 albums')
        self.assertContains(response, '6 songs')
        self.assertQuerysetEqual(response.context['albums'].data,
            [repr(al) for al in [reg_album_1, reg_album_2, misc_album, va_album]])
        self.assertQuerysetEqual(response.context['songs'].data,
            [repr(s) for s in Song.objects.filter(artist=artist).order_by('title')])

        # There are certainly some duplicate tests happening down here.
        for album in [reg_album_1, reg_album_2, misc_album, va_album]:
            self.assertContains(response, str(album))
            self.assertContains(response, str(album.artist))
            self.assertContains(response, reverse('exordium:album', args=(album.pk,)))
            self.assertContains(response, reverse('exordium:artist', args=(album.artist.normname,)))
        for song in Song.objects.filter(artist=artist):
            self.assertContains(response, str(song.title))
            self.assertContains(response, song.get_download_url_html5())
            self.assertContains(response, song.get_download_url_m3u())
        for song in Song.objects.exclude(artist=artist):
            self.assertNotContains(response, str(song.title))
            self.assertNotContains(response, song.get_download_url_html5())
            self.assertNotContains(response, song.get_download_url_m3u())

    def test_classical_as_conductor(self):
        """
        Test the scenario where our only track is as a conductor on
        a track on an album tagged with classical stuff.
        """
        self.add_mp3(artist='Artist 1', title='Title 1',
            group='Group 1', conductor='Conductor 1', composer='Composer 1',
            album='Album 1', filename='song1.mp3', path='album_1')
        self.add_mp3(artist='Artist 1', title='Title 2',
            group='Group 2', conductor='Conductor 2', composer='Composer 2',
            album='Album 1', filename='song2.mp3', path='album_1')
        self.run_add()

        self.assertEqual(Artist.objects.count(), 8)
        artist = Artist.objects.get(name='Conductor 2')

        self.assertEqual(Album.objects.count(), 1)
        album = Album.objects.get()

        self.assertEqual(Song.objects.count(), 2)
        song_1 = Song.objects.get(title='Title 1')
        song_2 = Song.objects.get(title='Title 2')

        response = self.client.get(reverse('exordium:artist', args=(artist.normname,)))
        self.assertEqual(response.status_code, 200)
        self.assertQuerysetEqual(response.context['albums'].data, [repr(album)])
        self.assertContains(response, '1 album')
        self.assertQuerysetEqual(response.context['songs'].data, [repr(song_2)])
        self.assertContains(response, '1 song')
        self.assertContains(response, str(album))
        self.assertContains(response, reverse('exordium:album', args=(album.pk,)))
        
        for artist in Artist.objects.exclude(name='Various'):
            self.assertContains(response, str(artist))
            self.assertContains(response, reverse('exordium:artist', args=(artist.normname,)))

        self.assertNotContains(response, str(song_1))
        self.assertNotContains(response, song_1.get_download_url_html5())
        self.assertNotContains(response, song_1.get_download_url_m3u())
        self.assertContains(response, str(song_2))
        self.assertContains(response, song_2.get_download_url_html5())
        self.assertContains(response, song_2.get_download_url_m3u())

    def test_classical_as_conductor_various(self):
        """
        Test the scenario where our only track is as a conductor on
        a track on an album tagged with classical stuff.  Also the
        album is classified as Various Artists
        """
        self.add_mp3(artist='Artist 1', title='Title 1',
            group='Group 1', conductor='Conductor 1', composer='Composer 1',
            album='Album 1', filename='song1.mp3', path='album_1')
        self.add_mp3(artist='Artist 2', title='Title 2',
            group='Group 2', conductor='Conductor 2', composer='Composer 2',
            album='Album 1', filename='song2.mp3', path='album_1')
        self.run_add()

        self.assertEqual(Artist.objects.count(), 9)
        artist = Artist.objects.get(name='Conductor 2')

        self.assertEqual(Album.objects.count(), 1)
        album = Album.objects.get()

        self.assertEqual(Song.objects.count(), 2)
        song_1 = Song.objects.get(title='Title 1')
        song_2 = Song.objects.get(title='Title 2')

        response = self.client.get(reverse('exordium:artist', args=(artist.normname,)))
        self.assertEqual(response.status_code, 200)
        self.assertQuerysetEqual(response.context['albums'].data, [repr(album)])
        self.assertContains(response, '1 album')
        self.assertQuerysetEqual(response.context['songs'].data, [repr(song_2)])
        self.assertContains(response, '1 song')
        self.assertContains(response, str(album))
        self.assertContains(response, reverse('exordium:album', args=(album.pk,)))
        
        for artist in Artist.objects.exclude(name='Artist 1'):
            self.assertContains(response, str(artist))
            self.assertContains(response, reverse('exordium:artist', args=(artist.normname,)))
        for artist in Artist.objects.filter(name='Artist 1'):
            self.assertNotContains(response, str(artist))
            self.assertNotContains(response, reverse('exordium:artist', args=(artist.normname,)))

        self.assertNotContains(response, str(song_1))
        self.assertNotContains(response, song_1.get_download_url_html5())
        self.assertNotContains(response, song_1.get_download_url_m3u())
        self.assertContains(response, str(song_2))
        self.assertContains(response, song_2.get_download_url_html5())
        self.assertContains(response, song_2.get_download_url_m3u())

    def test_pagination(self):
        """
        Test pagination.  To actually use assertNotContains we have to do both
        album and song pagination at the same time, so we'll go ahead and do that
        rather than splitting them into separate tests.  We show 50 albums and
        25 songs on the artist page.
        """
        for num in range(60):
            self.add_mp3(artist='Artist', title='Title %02d' % (num+1),
                album='Album %02d' % (num+1), filename='song%d.mp3' % (num+1))
        self.run_add()
        self.assertEqual(Album.objects.count(), 60)

        self.assertEqual(Artist.objects.count(), 2)
        artist = Artist.objects.get(name='Artist')

        albums = {}
        for num in range(60):
            albums[num] = Album.objects.get(name='Album %02d' % (num+1))

        songs = {}
        for num in range(60):
            songs[num] = Song.objects.get(title='Title %02d' % (num+1))

        response = self.client.get(reverse('exordium:artist', args=(artist.normname,)))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '50 of 60 albums')
        self.assertContains(response, '25 of 60 songs')
        self.assertContains(response, '"?album-page=2"')
        self.assertContains(response, '"?song-page=2"')
        self.assertEqual(len(response.context['albums'].data), 60)
        self.assertEqual(len(response.context['songs'].data), 60)
        for num in range(50):
            self.assertContains(response, '%s<' % (albums[num]))
            self.assertContains(response, reverse('exordium:album', args=(albums[num].pk,)))
        for num in range(50, 60):
            self.assertNotContains(response, '%s<' % (albums[num]))
            self.assertNotContains(response, reverse('exordium:album', args=(albums[num].pk,)))
        for num in range(25):
            self.assertContains(response, '%s<' % (songs[num]))
        for num in range(25, 60):
            self.assertNotContains(response, '%s<' % (songs[num]))

        # test page 2/3
        response = self.client.get(reverse('exordium:artist', args=(artist.normname,)), {'album-page': 2, 'song-page': 3})
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '10 of 60 albums')
        self.assertContains(response, '10 of 60 songs')
        self.assertContains(response, 'album-page=1')
        self.assertContains(response, 'song-page=2')
        self.assertEqual(len(response.context['albums'].data), 60)
        self.assertEqual(len(response.context['songs'].data), 60)
        for num in range(50):
            self.assertNotContains(response, '%s<' % (albums[num]))
            self.assertNotContains(response, reverse('exordium:album', args=(albums[num].pk,)))
        for num in range(50, 60):
            self.assertContains(response, '%s<' % (albums[num]))
            self.assertContains(response, reverse('exordium:album', args=(albums[num].pk,)))
        for num in range(50):
            self.assertNotContains(response, '%s<' % (songs[num]))
        for num in range(50, 60):
            self.assertContains(response, '%s<' % (songs[num]))

    def test_sorting_album(self):
        """
        Test at least one case of album sorting.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album 1', year=2016, filename='song1.mp3')
        self.add_mp3(artist='Artist', title='Title 2',
            album='Album 2', year=2006, filename='song2.mp3')
        self.add_mp3(artist='Artist', title='Title 3',
            album='Album 3', year=1996, filename='song3.mp3')
        self.run_add()
        self.assertEqual(Album.objects.count(), 3)

        albums = [
            Album.objects.get(name='Album 1'),
            Album.objects.get(name='Album 2'),
            Album.objects.get(name='Album 3'),
        ]
        artist = Artist.objects.get(name='Artist')

        response = self.client.get(reverse('exordium:artist', args=(artist.normname,)))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['albums'].data), 3)
        self.assertQuerysetEqual(response.context['albums'].data, [repr(al) for al in albums])
        self.assertContains(response, '"?album-sort=year"')

        # test the sorting button
        response = self.client.get(reverse('exordium:artist', args=(artist.normname,)), {'album-sort': 'year'})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['albums'].data), 3)
        self.assertQuerysetEqual(response.context['albums'].data, [repr(al) for al in reversed(albums)])
        self.assertContains(response, '"?album-sort=-year"')

    def test_sorting_album_year_time_added(self):
        """
        Test album sorting by year when two albums are in the same year; should
        use time_added as the secondary sorting field.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album 1', year=2017, filename='song1.mp3')
        self.add_mp3(artist='Artist', title='Title 2',
            album='Album 2', year=2017, filename='song2.mp3')
        self.run_add()
        al2 = self.age_album('Artist', 'Album 2', 10)
        self.assertEqual(Album.objects.count(), 2)

        albums = [
            al2,
            Album.objects.get(name='Album 1'),
        ]
        artist = Artist.objects.get(name='Artist')

        response = self.client.get(reverse('exordium:artist', args=(artist.normname,)), {'album-sort': 'year'})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['albums'].data), 2)
        self.assertQuerysetEqual(response.context['albums'].data, [repr(al) for al in albums])
        self.assertContains(response, '"?album-sort=-year"')

        # test reverse sort
        response = self.client.get(reverse('exordium:artist', args=(artist.normname,)), {'album-sort': '-year'})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['albums'].data), 2)
        self.assertQuerysetEqual(response.context['albums'].data, [repr(al) for al in reversed(albums)])
        self.assertContains(response, '"?album-sort=year"')

    def test_sorting_song(self):
        """
        Test at least one case of song sorting.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album 3', filename='song1.mp3')
        self.add_mp3(artist='Artist', title='Title 2',
            album='Album 2', filename='song2.mp3')
        self.add_mp3(artist='Artist', title='Title 3',
            album='Album 1', filename='song3.mp3')
        self.run_add()
        self.assertEqual(Album.objects.count(), 3)

        songs = [
            Song.objects.get(title='Title 1'),
            Song.objects.get(title='Title 2'),
            Song.objects.get(title='Title 3'),
        ]
        artist = Artist.objects.get(name='Artist')

        response = self.client.get(reverse('exordium:artist', args=(artist.normname,)))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['songs'].data), 3)
        self.assertQuerysetEqual(response.context['songs'].data, [repr(al) for al in songs])
        self.assertContains(response, '"?song-sort=album"')

        # test the sorting button
        response = self.client.get(reverse('exordium:artist', args=(artist.normname,)), {'song-sort': 'album'})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['songs'].data), 3)
        self.assertQuerysetEqual(response.context['songs'].data, [repr(al) for al in reversed(songs)])
        self.assertContains(response, '"?song-sort=-album"')

    def test_too_many_songs(self):
        """
        If an artist has more than 500 songs, the song list won't be
        shown on the artist page.  We're going to cheat here and
        insert directly into the database rather than going through our
        ``run_add()`` rigamarole.  Quite possibly I should be doing that
        on all our pagination tests, too, since those are noticeably
        slow.  Ah well.
        """
        artist = Artist.objects.create(name='Artist', normname='artist')
        album = Album.objects.create(artist=artist, name='Album', normname='album')
        for num in range(501):
            Song.objects.create(filename='file%03d.mp3' % (num+1),
                artist=artist,
                album=album,
                title='Title %03d' % (num+1),
                year=0,
                tracknum=0,
                normtitle='title %03d' % (num+1),
                raw_artist='artist',
                filetype=Song.MP3,
                bitrate=128000,
                mode=Song.CBR,
                size=123000,
                length=90,
                sha256sum='0cf31fc7d968ec16c69758f9b0ebb2355471d5694a151b40e5e4f8641b061092',
            )

        response = self.client.get(reverse('exordium:artist', args=(artist.normname,)))
        self.assertEqual(response.status_code, 200)
        self.assertQuerysetEqual(response.context['albums'].data, [repr(album)])
        self.assertEqual(response.context['have_songs'], False)
        self.assertNotIn('songs', response.context)
        self.assertNotContains(response, 'Songs by %s' % (artist))
        self.assertContains(response, reverse('exordium:artist', args=(artist.normname,)))
        self.assertContains(response, reverse('exordium:album', args=(album.pk,)))
        self.assertContains(response, '1 album')
        self.assertNotContains(response, '1 song')




from .base import ExordiumUserTests

from django.test import TestCase
from django.urls import reverse

import io

from PIL import Image

from exordium.models import Artist, Album, Song, App, AlbumArt

class AlbumArtTests(ExordiumUserTests):
    """
    Tests about album art specifically
    """

    def test_basic_add_album_art_gif(self):
        """
        Test a simple case where we have a gif album art.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art(basefile='cover_400.gif', filename='cover.gif')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, 'cover.gif')
        self.assertEqual(al.art_mime, 'image/gif')
        self.assertEqual(al.art_ext, 'gif')

    def test_basic_add_album_art_png(self):
        """
        Test a simple case where we have a png album art.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art(basefile='cover_400.png', filename='cover.png')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, 'cover.png')
        self.assertEqual(al.art_mime, 'image/png')
        self.assertEqual(al.art_ext, 'png')

    def test_basic_add_album_art_jpg(self):
        """
        Test a simple case where we have a jpg album art.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art(basefile='cover_400.jpg', filename='cover.jpg')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, 'cover.jpg')
        self.assertEqual(al.art_mime, 'image/jpeg')
        self.assertEqual(al.art_ext, 'jpg')

    def test_basic_add_album_art_without_filesystem_permissions(self):
        """
        Test a simple case where we have album art but the filesystem
        permissions are bad.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art(filename='cover.jpg')
        self.set_file_unreadable('cover.jpg')
        self.run_add_errors(error='found but not readable')

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.has_album_art(), False)

    def test_basic_add_album_art_miscellaneous(self):
        """
        Test a case where a miscellaneous album has an image cover.  The
        image should not get loaded.
        """
        self.add_mp3(artist='Artist', title='Title 1', filename='song1.mp3')
        self.add_art()
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.miscellaneous, True)
        self.assertEqual(al.has_album_art(), False)
        self.assertEqual(al.art_filename, None)

    def test_basic_update_add_album_art(self):
        """
        Test a simple case where we add album art during an update,
        rather than in the add.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.run_add()

        self.add_art(basefile='cover_400.jpg', filename='cover.jpg')
        self.run_update()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, 'cover.jpg')
        self.assertEqual(al.art_mime, 'image/jpeg')
        self.assertEqual(al.art_ext, 'jpg')

    def test_basic_update_add_album_art_without_filesystem_permissions(self):
        """
        Test a simple case where we add album art during an update,
        rather than in the add.  The filesystem permissions are bad
        on the cover, though.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.run_add()

        self.add_art(filename='cover.jpg')
        self.set_file_unreadable('cover.jpg')
        self.run_update_errors(error='found but not readable')

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.has_album_art(), False)

    def test_basic_add_album_art_parent_dir(self):
        """
        Tests associating an album cover which is in the song's parent dir.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3', path='Album')
        self.add_art(basefile='cover_400.jpg', filename='cover.jpg')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, 'cover.jpg')
        self.assertEqual(al.art_mime, 'image/jpeg')
        self.assertEqual(al.art_ext, 'jpg')

    def test_basic_add_album_art_two_dirs_up(self):
        """
        Tests associating an album cover which is two directories
        above - or in other words should NOT be found.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3', path='Artist/Album')
        self.add_art(basefile='cover_400.jpg', filename='cover.jpg')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, None)

    def test_basic_add_album_art_mismatched_extension(self):
        """
        Tests associating an album cover which has an improper
        extension.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art(basefile='cover_400.png', filename='cover.jpg')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, 'cover.jpg')
        self.assertEqual(al.art_mime, 'image/png')
        self.assertEqual(al.art_ext, 'png')

    def test_basic_add_album_art_invalid_file(self):
        """
        Test a cover file which isn't actually an image we support.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art(basefile='silence-vbr.mp3', filename='cover.jpg')
        self.run_add_errors(error='cannot identify image file')

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, None)
        self.assertEqual(al.art_mime, None)
        self.assertEqual(al.art_ext, None)
        self.assertEqual(al.art_mtime, 0)

    def test_album_art_removal(self):
        """
        Tests removal of album art during an update()
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art()
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, 'cover.jpg')

        self.delete_file('cover.jpg')
        self.run_update()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, None)

    def test_non_album_art(self):
        """
        Tests what happens when there's an image alongside a non-album
        track.  The art should NOT be associated!  The special "non-album"
        albums don't get album art.
        """
        self.add_mp3(artist='Artist', title='Title 1', filename='song1.mp3')
        self.add_art()
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, None)

    def test_album_art_update_no_changes(self):
        """
        Tests what happens on an update when the album art hasn't changed.
        Nothing should change in the record.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art()
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, 'cover.jpg')
        orig_mtime = al.art_mtime
        orig_filename = al.art_filename
        orig_mime = al.art_mime
        orig_ext = al.art_ext

        self.run_update()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, 'cover.jpg')
        self.assertEqual(orig_mtime, al.art_mtime)
        self.assertEqual(orig_filename, al.art_filename)
        self.assertEqual(orig_mime, al.art_mime)
        self.assertEqual(orig_ext, al.art_ext)

    def test_album_art_update_mtime(self):
        """
        Tests what happens when an album art file is updated
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art()
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, 'cover.jpg')
        orig_mtime = al.art_mtime

        self.touch_file('cover.jpg')
        self.run_update()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, 'cover.jpg')
        self.assertNotEqual(orig_mtime, al.art_mtime)

    def test_album_art_update_different_file(self):
        """
        Tests what happens when an album art file changes filenames.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art(filename='cover-orig.jpg')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, 'cover-orig.jpg')

        self.delete_file('cover-orig.jpg')
        self.add_art(filename='cover-new.jpg')
        self.run_update()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, 'cover-new.jpg')

    def test_album_art_update_better_filename(self):
        """
        Tests what happens when a new album art file turns up during
        an update which we consider "better" than the one we're using.
        (ie: moving from GIF to PNG, or moving from "whatever.jpg" to
        "cover.jpg").  Nothing should happen, since the regular update
        will only do something if the mtime on the existing file changes,
        or the file disappears.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art(filename='blah.jpg')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, 'blah.jpg')

        self.add_art(filename='cover.jpg')
        self.run_update()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, 'blah.jpg')

    def test_album_art_update_better_filename_direct(self):
        """
        Tests what happens when a new album art file turns up during
        an update which we consider "better" than the one we're using.
        (ie: moving from GIF to PNG, or moving from "whatever.jpg" to
        "cover.jpg").  As with the previous test, we expect nothing to
        change, but instead of going through a full update we're calling
        Album.update_album_art() directly.  This shouldn't be any
        different unless we also pass in full_refresh=True to that
        function, so the album art should remain.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art(filename='blah.jpg')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, 'blah.jpg')

        self.add_art(filename='cover.jpg')
        self.assertNoErrors(list(al.update_album_art()))

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, 'blah.jpg')

    def test_album_art_update_better_filename_forced(self):
        """
        Tests what happens when a new album art file turns up during
        an update which we consider "better" than the one we're using.
        (ie: moving from GIF to PNG, or moving from "whatever.jpg" to
        "cover.jpg").  This time we are going to simulate a force of
        that update.  Ordinarily this'll only happen direct from an
        admin page click, and is a separate action from add() and update()
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art(filename='blah.jpg')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, 'blah.jpg')

        self.add_art(filename='cover.jpg')
        self.assertNoErrors(list(al.update_album_art(full_refresh=True)))

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, 'cover.jpg')

    def test_album_art_update_miscellaneous_to_regular(self):
        """
        Testing what happens when a miscellaneous album (therefore without
        album art) is updated to be a "real" album - it should find the 
        album art present.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            filename='song1.mp3')
        self.add_art()
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.miscellaneous, True)
        self.assertEqual(al.has_album_art(), False)

        self.update_mp3('song1.mp3', album='Album')
        self.run_update()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.miscellaneous, False)
        self.assertEqual(al.has_album_art(), True)
        self.assertEqual(al.art_filename, 'cover.jpg')

    def test_album_art_update_regular_to_miscellaneous(self):
        """
        Testing what happens when a regular album (with album art)
        is updated to be a miscellaneous album.  The album art info
        should be discarded.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art()
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.miscellaneous, False)
        self.assertEqual(al.has_album_art(), True)
        self.assertEqual(al.art_filename, 'cover.jpg')

        self.update_mp3('song1.mp3', album='')
        self.run_update()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.miscellaneous, True)
        self.assertEqual(al.has_album_art(), False)
        self.assertEqual(al.art_filename, None)

    def test_album_art_move_to_different_directory(self):
        """
        Testing what happens when a regular album (with album art)
        is moved to a different directory.  The album art info
        should be moved along with it.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3', path='Old')
        self.add_art(path='Old')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.has_album_art(), True)
        self.assertEqual(al.art_filename, 'Old/cover.jpg')

        self.move_file('Old/song1.mp3', 'New')
        self.move_file('Old/cover.jpg', 'New')
        self.run_update()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.has_album_art(), True)
        self.assertEqual(al.art_filename, 'New/cover.jpg')

    def test_album_art_move_to_different_directory_and_leave_art(self):
        """
        Testing what happens when a regular album (with album art)
        is moved to a different directory, but the album art is not
        moved along with it.  In this case the album art remains where
        it is, and though it seems a bit counterintuitive, we'll permit
        the album art records to stay where they are.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3', path='Old')
        self.add_art(path='Old')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.has_album_art(), True)
        self.assertEqual(al.art_filename, 'Old/cover.jpg')

        self.move_file('Old/song1.mp3', 'New')
        self.run_update()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.has_album_art(), True)
        self.assertEqual(al.art_filename, 'Old/cover.jpg')

    def test_album_art_move_to_different_directory_and_remove_art(self):
        """
        Testing what happens when a regular album (with album art)
        is moved to a different directory, but the album art is
        deleted.  In this case the album art info should be removed.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3', path='Old')
        self.add_art(path='Old')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.has_album_art(), True)
        self.assertEqual(al.art_filename, 'Old/cover.jpg')

        self.move_file('Old/song1.mp3', 'New')
        self.delete_file('Old/cover.jpg')
        self.run_update()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.has_album_art(), False)
        self.assertEqual(al.art_filename, None)

    def test_album_art_view_retrieve_original_jpg(self):
        """
        Test retrieval of the fullsize cover art from a URL
        """
        self.longMessage = False

        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art(basefile='cover_400.jpg', filename='cover.jpg')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        url = reverse('exordium:origalbumart', args=(al.pk, al.art_ext))
        response = self.client.get(url)
        filedata = self.get_file_contents('cover.jpg')

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'image/jpeg')
        self.assertEqual(response.content, filedata, 'File data differs')

    def test_album_art_view_retrieve_original_gif(self):
        """
        Test retrieval of the fullsize cover art from a URL
        """
        self.longMessage = False

        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art(basefile='cover_400.gif', filename='cover.gif')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        url = reverse('exordium:origalbumart', args=(al.pk, al.art_ext))
        response = self.client.get(url)
        filedata = self.get_file_contents('cover.gif')

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'image/gif')
        self.assertEqual(response.content, filedata, 'File data differs')

    def test_album_art_view_retrieve_original_png(self):
        """
        Test retrieval of the fullsize cover art from a URL
        """
        self.longMessage = False

        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art(basefile='cover_400.png', filename='cover.png')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        url = reverse('exordium:origalbumart', args=(al.pk, al.art_ext))
        response = self.client.get(url)
        filedata = self.get_file_contents('cover.png')

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'image/png')
        self.assertEqual(response.content, filedata, 'File data differs')

    def test_album_art_view_retrieve_original_no_cover(self):
        """
        Test retrieval of the fullsize cover art from a URL, when we don't
        actually have an image to return.  Should throw a 404 (our "no
        album art found" graphic is displayed via templates, not code)
        """
        self.longMessage = False

        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, None)
        url = reverse('exordium:origalbumart', args=(al.pk, 'jpg'))
        response = self.client.get(url)

        self.assertEqual(response.status_code, 404)

    def test_album_art_view_retrieve_original_extension_mismatch(self):
        """
        Test retrieval of the fullsize cover art from a URL, with
        a mismatched extension on the original filename
        """
        self.longMessage = False

        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art(basefile='cover_400.jpg', filename='cover.png')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        url = reverse('exordium:origalbumart', args=(al.pk, al.art_ext))
        response = self.client.get(url)
        filedata = self.get_file_contents('cover.png')

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'image/jpeg')
        self.assertEqual(response.content, filedata, 'File data differs')

    def test_album_art_generate_album_thumb(self):
        """
        Test the creation of an album-sized thumbnail for our art
        """
        self.longMessage = False

        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art()
        self.run_add()

        size = AlbumArt.SZ_ALBUM
        resolution = AlbumArt.resolutions[size]

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        url = reverse('exordium:albumart', args=(al.pk, size))
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'image/jpeg')

        data = io.BytesIO(response.content)
        im = Image.open(data)
        self.assertEqual(im.width, resolution)
        self.assertEqual(im.height, resolution)

        data.seek(0)
        self.assertEqual(AlbumArt.objects.count(), 1)
        art = AlbumArt.objects.get(album=al, size=size)
        self.assertEqual(art.resolution, resolution)
        self.assertEqual(art.from_mtime, al.art_mtime)
        self.assertEqual(bytes(art.image), data.read())

        # Test our get_artist() method, for coverage.py
        ar = Artist.objects.get(name='Artist')
        self.assertEqual(art.get_artist(), ar)

    def test_album_art_generate_album_thumb_twice(self):
        """
        Test the creation of an album-sized thumbnail for our art,
        and call twice.  Mostly just to increase some coverage via
        coverage.py, to ensure we're hitting the bit which returns
        and existing object.
        """
        self.longMessage = False

        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art()
        self.run_add()

        size = AlbumArt.SZ_ALBUM
        resolution = AlbumArt.resolutions[size]

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        url = reverse('exordium:albumart', args=(al.pk, size))
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'image/jpeg')

        data = io.BytesIO(response.content)
        im = Image.open(data)
        self.assertEqual(im.width, resolution)
        self.assertEqual(im.height, resolution)

        data.seek(0)
        self.assertEqual(AlbumArt.objects.count(), 1)
        art = AlbumArt.objects.get(album=al, size=size)
        art_pk = art.pk
        self.assertEqual(art.resolution, resolution)
        self.assertEqual(art.from_mtime, al.art_mtime)
        self.assertEqual(bytes(art.image), data.read())

        # Now the second request - may as well do all the same checks
        # one more time.
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'image/jpeg')

        data = io.BytesIO(response.content)
        im = Image.open(data)
        self.assertEqual(im.width, resolution)
        self.assertEqual(im.height, resolution)

        data.seek(0)
        self.assertEqual(AlbumArt.objects.count(), 1)
        art = AlbumArt.objects.get(album=al, size=size)
        self.assertEqual(art.pk, art_pk)
        self.assertEqual(art.resolution, resolution)
        self.assertEqual(art.from_mtime, al.art_mtime)
        self.assertEqual(bytes(art.image), data.read())

    def test_album_art_generate_album_thumb_gif(self):
        """
        Test the creation of an album-sized thumbnail for our art,
        using a GIF instead of JPG.
        """
        self.longMessage = False

        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art(basefile='cover_400.gif', filename='cover.gif')
        self.run_add()

        size = AlbumArt.SZ_ALBUM
        resolution = AlbumArt.resolutions[size]

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        url = reverse('exordium:albumart', args=(al.pk, size))
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'image/jpeg')

        data = io.BytesIO(response.content)
        im = Image.open(data)
        self.assertEqual(im.width, resolution)
        self.assertEqual(im.height, resolution)

        data.seek(0)
        self.assertEqual(AlbumArt.objects.count(), 1)
        art = AlbumArt.objects.get(album=al, size=size)
        self.assertEqual(art.resolution, resolution)
        self.assertEqual(art.from_mtime, al.art_mtime)
        self.assertEqual(bytes(art.image), data.read())

    def test_album_art_generate_album_thumb_png(self):
        """
        Test the creation of an album-sized thumbnail for our art,
        using a PNG instead of JPG.
        """
        self.longMessage = False

        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art(basefile='cover_400.png', filename='cover.png')
        self.run_add()

        size = AlbumArt.SZ_ALBUM
        resolution = AlbumArt.resolutions[size]

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        url = reverse('exordium:albumart', args=(al.pk, size))
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'image/jpeg')

        data = io.BytesIO(response.content)
        im = Image.open(data)
        self.assertEqual(im.width, resolution)
        self.assertEqual(im.height, resolution)

        data.seek(0)
        self.assertEqual(AlbumArt.objects.count(), 1)
        art = AlbumArt.objects.get(album=al, size=size)
        self.assertEqual(art.resolution, resolution)
        self.assertEqual(art.from_mtime, al.art_mtime)
        self.assertEqual(bytes(art.image), data.read())

    def test_album_art_generate_list_thumb(self):
        """
        Test the creation of a list-sized thumbnail for our art
        """
        self.longMessage = False

        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art()
        self.run_add()

        size = AlbumArt.SZ_LIST
        resolution = AlbumArt.resolutions[size]

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        url = reverse('exordium:albumart', args=(al.pk, size))
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'image/jpeg')

        data = io.BytesIO(response.content)
        im = Image.open(data)
        self.assertEqual(im.width, resolution)
        self.assertEqual(im.height, resolution)

        data.seek(0)
        self.assertEqual(AlbumArt.objects.count(), 1)
        art = AlbumArt.objects.get(album=al, size=size)
        self.assertEqual(art.resolution, resolution)
        self.assertEqual(art.from_mtime, al.art_mtime)
        self.assertEqual(bytes(art.image), data.read())

    def test_album_art_attempt_invalid_thumb_generation(self):
        """
        Test the creation of an album art thumbnail for a type of
        thumbnail we don't actually support.  Should generate a 404.
        """

        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art()
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, 'cover.jpg')
        url = reverse('exordium:albumart', args=(al.pk, 'foobar'))
        response = self.client.get(url)

        self.assertEqual(response.status_code, 404)
        self.assertEqual(AlbumArt.objects.count(), 0)

    def test_album_art_model_get_or_create_invalid_size(self):
        """
        Test the creation of an album art thumbnail for a type of
        thumbnail we don't actually support.  This time going after the
        actual model method which does the work.  In reality it couldn't
        get this far because we've already checked, but just in case
        we'll make sure we have this test here as well.
        """

        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art()
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, 'cover.jpg')
        self.assertEqual(AlbumArt.get_or_create(al, 'foobar'), None)
        self.assertEqual(AlbumArt.objects.count(), 0)

    def test_album_art_attempt_unknown_album_thumb_retrieval(self):
        """
        Test the retrieval of an album art thumbnail for an album
        which doesn't exist.  Should generate a 404.
        """
        self.assertEqual(Album.objects.count(), 0)
        url = reverse('exordium:albumart', args=(42, AlbumArt.SZ_ALBUM))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)
        self.assertEqual(AlbumArt.objects.count(), 0)

    def test_album_art_attempt_album_thumb_generation_without_art(self):
        """
        Test the creation of an album art thumbnail for an album which
        doesn't have album art.  Will throw a 404.  (Our "no album art
        found" graphic is shown via templates, not code.)
        """

        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, None)
        url = reverse('exordium:albumart', args=(al.pk, AlbumArt.SZ_ALBUM))
        response = self.client.get(url)

        self.assertEqual(response.status_code, 404)

    def test_album_art_resolution_preset_change(self):
        """
        Test what happens when we have an thumbnail but then our
        preset resolution size changes.  The thumbnail should get
        regenerated when it's re-requested.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art()
        self.run_add()

        size = AlbumArt.SZ_ALBUM
        resolution = AlbumArt.resolutions[size]

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, 'cover.jpg')

        url = reverse('exordium:albumart', args=(al.pk, size))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'image/jpeg')

        orig_data = io.BytesIO(response.content)
        im = Image.open(orig_data)
        self.assertEqual(im.width, resolution)
        self.assertEqual(im.height, resolution)

        self.assertEqual(AlbumArt.objects.count(), 1)
        art = AlbumArt.objects.get()
        self.assertEqual(art.resolution, resolution)
        art_pk = art.pk

        AlbumArt.resolutions[size] = resolution - 50
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'image/jpeg')

        new_data = io.BytesIO(response.content)
        im = Image.open(new_data)
        self.assertEqual(im.width, resolution - 50)
        self.assertEqual(im.height, resolution - 50)

        orig_data.seek(0)
        new_data.seek(0)
        self.assertNotEqual(orig_data, new_data)

        self.assertEqual(AlbumArt.objects.count(), 1)
        art = AlbumArt.objects.get(album=al, size=size)
        self.assertEqual(art.resolution, resolution - 50)
        self.assertNotEqual(art_pk, art.pk)

        # Reset the resolutions dict, though we also do this in
        # the test tearDown, in case our test fails before we
        # get here.
        AlbumArt.resolutions[size] = resolution

    def test_album_art_get_art_after_file_deletion(self):
        """
        Test what happens when we have album art but the file is
        deleted before we get a chance to request one of the
        thumbnails.  The album art should get removed and we
        should get no art back.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art()
        self.run_add()

        self.delete_file('cover.jpg')

        size = AlbumArt.SZ_ALBUM
        resolution = AlbumArt.resolutions[size]

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.has_album_art(), True)
        self.assertEqual(al.art_filename, 'cover.jpg')

        url = reverse('exordium:albumart', args=(al.pk, size))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)

        # Get the album again and run some tests
        al = Album.objects.get()
        self.assertEqual(al.has_album_art(), False)
        self.assertEqual(al.art_filename, None)

    def test_album_art_update_after_source_update(self):
        """
        Test what happens when we have an thumbnail but then our
        preset resolution size changes.  The thumbnail should get
        regenerated when it's re-requested.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art()
        self.run_add()

        size = AlbumArt.SZ_ALBUM
        resolution = AlbumArt.resolutions[size]

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, 'cover.jpg')

        url = reverse('exordium:albumart', args=(al.pk, size))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'image/jpeg')

        orig_data = io.BytesIO(response.content)
        im = Image.open(orig_data)
        self.assertEqual(im.width, resolution)
        self.assertEqual(im.height, resolution)

        self.assertEqual(AlbumArt.objects.count(), 1)
        art = AlbumArt.objects.get()
        self.assertEqual(art.resolution, resolution)
        art_pk = art.pk

        self.touch_file('cover.jpg')
        self.run_update()

        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'image/jpeg')

        new_data = io.BytesIO(response.content)
        im = Image.open(new_data)
        self.assertEqual(im.width, resolution)
        self.assertEqual(im.height, resolution)

        orig_data.seek(0)
        new_data.seek(0)
        self.assertNotEqual(orig_data, new_data)

        self.assertEqual(AlbumArt.objects.count(), 1)
        art = AlbumArt.objects.get(album=al, size=size)
        self.assertEqual(art.resolution, resolution)
        self.assertNotEqual(art_pk, art.pk)

    def test_admin_update_without_permission(self):
        """
        Test an admin album art update when we're not actually logged in.
        """

        url = reverse('exordium:albumartupdate', args=(42,))
        response = self.client.get(url)
        self.assertRedirects(response, '%s?next=%s' % (reverse('admin:login'), url))

    def test_admin_update_invalid_album(self):
        """
        Test an admin album art update for an invalid album ID
        """

        self.login()
        response = self.client.get(reverse('exordium:albumartupdate', args=(42,)))
        self.assertEqual(response.status_code, 404)

    def test_admin_update_no_change(self):
        """
        Test an admin album art update for an album which doesn't actually change.
        """

        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art()
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, 'cover.jpg')
        orig_mtime = al.art_mtime
        orig_filename = al.art_filename
        orig_mime = al.art_mime
        orig_ext = al.art_ext

        self.login()
        response = self.client.get(reverse('exordium:albumartupdate', args=(al.pk,)))
        self.assertRedirects(response, reverse('exordium:album', args=(al.pk,)))

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, 'cover.jpg')
        self.assertEqual(orig_mtime, al.art_mtime)
        self.assertEqual(orig_filename, al.art_filename)
        self.assertEqual(orig_mime, al.art_mime)
        self.assertEqual(orig_ext, al.art_ext)

    def test_admin_update_better_filename(self):
        """
        Tests what happens when an update sees a "better" cover filename.
        Our admin update should pick up on it.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art(filename='blah.jpg')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, 'blah.jpg')

        self.add_art(filename='cover.jpg')

        self.login()
        response = self.client.get(reverse('exordium:albumartupdate', args=(al.pk,)))
        self.assertRedirects(response, reverse('exordium:album', args=(al.pk,)))

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, 'cover.jpg')

    def test_admin_update_file_permissions_wrong_no_initial_art(self):
        """
        Test an admin album art update in which the new album art file
        isn't actually readable.  (When the album did not have art,
        initially.)
        """

        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.has_album_art(), False)

        self.add_art(filename='cover.jpg')
        self.set_file_unreadable('cover.jpg')

        self.login()
        response = self.client.get(reverse('exordium:albumartupdate', args=(al.pk,)))
        self.assertRedirects(response, reverse('exordium:album', args=(al.pk,)),
            fetch_redirect_response=False)

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.has_album_art(), False)

        # Check the next page to ensure we got an error message
        response = self.client.get(reverse('exordium:album', args=(al.pk,)))
        self.assertContains(response, 'found but not readable')

    def test_admin_update_no_change_with_initial_art(self):
        """
        Test an admin album art update for an album which doesn't actually change.
        In this scenario we did have art for the album initially.
        """

        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art(filename='cover.jpg')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, 'cover.jpg')
        orig_mtime = al.art_mtime
        orig_filename = al.art_filename
        orig_mime = al.art_mime
        orig_ext = al.art_ext

        self.set_file_unreadable('cover.jpg')

        self.login()
        response = self.client.get(reverse('exordium:albumartupdate', args=(al.pk,)))
        self.assertRedirects(response, reverse('exordium:album', args=(al.pk,)),
            fetch_redirect_response=False)

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.art_filename, 'cover.jpg')
        self.assertEqual(orig_mtime, al.art_mtime)
        self.assertEqual(orig_filename, al.art_filename)
        self.assertEqual(orig_mime, al.art_mime)
        self.assertEqual(orig_ext, al.art_ext)

        # Check the next page to ensure we got an error message
        response = self.client.get(reverse('exordium:album', args=(al.pk,)))
        self.assertContains(response, 'found but not readable')

    def test_model_get_album_image_miscellaneous(self):
        """
        Ensure that Album.get_album_image() returns None when the album is
        set to miscellaneous, regardless of if an album image is set.  This
        can currently never actually happen in the app because we do that
        check as soon as possible, but we'll check the method here, regardless.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            filename='song1.mp3')
        self.add_art(filename='cover.jpg')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.miscellaneous, True)
        self.assertEqual(al.has_album_art(), False)
        self.assertEqual(al.get_album_image(), None)

        # Just for fun, pretend some album info made it into the database
        # somehow and ensure that our call still returns None.
        al.art_filename = 'cover.jpg'
        al.art_mime = 'media/jpeg'
        al.art_ext = 'jpg'
        al.art_mtime = 0
        al.save()

        self.assertEqual(al.has_album_art(), True)
        self.assertEqual(al.get_album_image(), None)

    def test_model_get_album_image_no_tracks(self):
        """
        Ensure that Album.get_album_image() returns None when we encounter an
        album with no tracks.  This shouldn't be possible under ordinary
        operation, but check for it anyway.  We will manually inject into the
        database to simulate this.
        """
        ar = Artist.objects.create(name='Artist', normname='artist')
        al = Album.objects.create(
            artist = ar,
            name = 'Album',
            normname = 'album',
            art_filename = 'cover.jpg',
            art_ext = 'jpg',
            art_mime = 'media/jpeg',
            art_mtime = 0,
        )

        self.assertEqual(al.has_album_art(), True)
        self.assertEqual(al.get_album_image(), None)

    def test_model_update_album_art_miscellaneous(self):
        """
        Ensure that ``Album.update_album_art()`` does nothing when the album is
        set to miscellaneous.  I'm pretty sure this can't ever actually happen
        in real life due to checks which occur prior to calling this, but
        we're checking here regardless.  We're passing ``full_refresh=True``
        to the procedure to ensure that we get right to the main bit.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            filename='song1.mp3')
        self.add_art(filename='cover.jpg')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.miscellaneous, True)
        self.assertEqual(al.has_album_art(), False)
        self.assertEqual(list(al.update_album_art(full_refresh=True)), [])
        self.assertEqual(al.get_album_image(), None)
        self.assertEqual(al.art_filename, None)
        self.assertEqual(al.art_ext, None)
        self.assertEqual(al.art_mime, None)
        self.assertEqual(al.art_mtime, 0)

    def test_model_import_album_image_from_filename_miscellaneous(self):
        """
        Ensure that Album.import_album_image_from_filename() does nothing
        when the album is set to miscellaneous.  This can currently
        never actually happen in the app because we do that check as soon
        as possible, but we'll check the method here, regardless.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            filename='song1.mp3')
        self.add_art(filename='cover.jpg')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.miscellaneous, True)
        self.assertEqual(al.has_album_art(), False)
        self.assertEqual(list(al.import_album_image_from_filename('cover.jpg', 'cover.jpg')), [])
        self.assertEqual(al.has_album_art(), False)
        self.assertEqual(al.art_filename, None)
        self.assertEqual(al.art_ext, None)
        self.assertEqual(al.art_mime, None)
        self.assertEqual(al.art_mtime, 0)

    def test_model_import_album_image_from_filename_invalid_extension(self):
        """
        Ensure that Album.import_album_image_from_filename() produces
        an error when the passed-in filename is of an invalid extension.
        This can currently never actually happen in the app because we do
        that check as soon as possible, but we'll check the method here,
        regardless.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art(filename='cover.pdf')
        self.run_add()

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.has_album_art(), False)
        self.assertErrors(list(al.import_album_image_from_filename('cover.pdf', 'cover.pdf')),
            error='Invalid extension for image')

    def test_add_image_with_invalid_internal_type(self):
        """
        Here we try importing album art which has a valid extension but is
        in reality a .tif file.
        """
        self.add_mp3(artist='Artist', title='Title 1',
            album='Album', filename='song1.mp3')
        self.add_art(filename='cover.jpg', basefile='cover_400.tif')
        self.run_add_errors(error='Unknown image type found')

        self.assertEqual(Album.objects.count(), 1)
        al = Album.objects.get()
        self.assertEqual(al.has_album_art(), False)

class BasicAlbumArtTests(TestCase):
    """
    Album art related tests which don't actually require our full
    fake library test setup, since they don't use add() or update().
    """

    def test_album_art_ordering(self):
        """
        Various tests for finding out if our album art preference sorting
        works how we hope.
        """
        order_tests = [
                (['cover.gif', 'cover.png'], ['cover.png', 'cover.gif']),
                (['cover.jpg', 'cover.png'], ['cover.png', 'cover.jpg']),
                (['cover-back.jpg', 'cover.jpg'], ['cover.jpg', 'cover-back.jpg']),
                (['cover-back.png', 'cover.jpg'], ['cover.jpg', 'cover-back.png']),
                (['cover-test.png', 'cover.gif'], ['cover.gif', 'cover-test.png']),
                (['other.png', 'cover.gif'], ['cover.gif', 'other.png']),
                (['other.png', 'cover-back.jpg'], ['cover-back.jpg', 'other.png']),
                (['cover-back.gif', 'cover-back.jpg'], ['cover-back.jpg', 'cover-back.gif']),
                (['cover-back.jpg', 'cover-back.png'], ['cover-back.png', 'cover-back.jpg']),
                (['zzz.jpg', 'aaa.jpg'], ['aaa.jpg', 'zzz.jpg']),
                (['zzz.png', 'aaa.jpg'], ['zzz.png', 'aaa.jpg']),
                (['foo.mp3', 'cover.png'], ['cover.png']),
                (['foo.mp3', 'cover.png', 'bar.txt'], ['cover.png']),
                (['foo.mp3', 'baz.gif', 'bar.txt'], ['baz.gif']),
                (['foo.mp3', 'baz.gif', 'bar.txt', 'frobozz.png'], ['frobozz.png', 'baz.gif']),
                (['foo.mp3', 'baz.aiff', 'bar.txt', 'frobozz.ogg'], []),
            ]

        # Loop through and test!
        for (before, after) in order_tests:
            self.assertEqual(App.get_cover_images(before), after)




# Generated by Django 4.0.2 on 2022-02-28 18:54

from django.db import migrations

def do_media_url_split(apps, schema_editor):  # pragma: no cover
    GlobalPreferenceModel = apps.get_model('dynamic_preferences', 'GlobalPreferenceModel')

    # Grab the existing prefs from the DB
    try:
        url_orig = GlobalPreferenceModel.objects.get(section='exordium', name='media_url')
    except GlobalPreferenceModel.DoesNotExist:
        url_orig = None
    try:
        url_html5 = GlobalPreferenceModel.objects.get(section='exordium', name='media_url_html5')
    except GlobalPreferenceModel.DoesNotExist:
        url_html5 = None
    try:
        url_m3u = GlobalPreferenceModel.objects.get(section='exordium', name='media_url_m3u')
    except GlobalPreferenceModel.DoesNotExist:
        url_m3u = None

    if not url_html5 and not url_m3u and url_orig:
        # If we don't have the new prefs but *do* have the orig, copy them over
        # and remove the orig
        url_html5 = GlobalPreferenceModel(
                section='exordium',
                name='media_url_html5',
                raw_value=url_orig.raw_value,
                )
        url_html5.save()
        url_m3u = GlobalPreferenceModel(
                section='exordium',
                name='media_url_m3u',
                raw_value=url_orig.raw_value,
                )
        url_m3u.save()
        url_orig.delete()

    elif url_html5 and url_m3u and url_orig:
        # If we have *all three* of them, just remove the orig.  I suspect this
        # could happen if the site is brought up before migrations have been
        # run, which would populate the new URLs with defaults.  The user may
        # very well have updated the URLs already, too, so deleting the orig
        # is the only sensible thing.
        url_orig.delete()

    else:
        # I think about the only other case possible would be that the app's never
        # been hit (in which case the prefs don't exist in the DB yet, and will be
        # populated by defaults when that happens).  So, do nothing.
        pass

class Migration(migrations.Migration):

    dependencies = [
        ('exordium', '0003_add_opus_support'),
        ('dynamic_preferences', '0006_auto_20191001_2236'),
    ]

    operations = [
        migrations.RunPython(do_media_url_split),
    ]

